import sys
import dcs
import dcs.mission
import dcs.task
import dcs.mapping
import dcs.point
from dcs.mapping import Point, Polygon
import dcs.terrain
import dcs.unittype
import dcs.vehicles
from dcs.countries import USA, Russia
import random
import argparse
import os
import datetime
from typing import List, Dict, Tuple


class BasicScenario:
    battle_zones = {
        "zugidi": Polygon([Point(-270285.71428571, 609257.14285714), Point(-264342.85714286, 614142.85714286),
                           Point(-257428.57142857, 619485.71428571), Point(-250857.14285714, 625171.42857143),
                           Point(-248342.85714286, 627457.14285714), Point(-244600, 628800),
                           Point(-238685.71428571, 635371.42857143), Point(-235257.14285714, 639942.85714285),
                           Point(-238685.71428571, 640771.42857143), Point(-239342.85714286, 642714.28571428),
                           Point(-235971.42857143, 644742.85714285), Point(-236600, 646142.85714285),
                           Point(-234085.71428571, 650142.85714285), Point(-239200, 656428.57142857),
                           Point(-249828.57142857, 653000), Point(-264400, 650000),
                           Point(-264828.57142857, 644171.42857142),
                           Point(-269885.71428571, 634542.85714285), Point(-273600, 635428.57142857),
                           Point(-273657.14285714, 642571.42857142), Point(-285771.42857143, 640828.57142857),
                           Point(-285800, 625514.28571428), Point(-279771.42857143, 625285.71428571),
                           Point(-273942.85714286, 629371.42857142), Point(-272428.57142857, 624828.57142857),
                           Point(-273342.85714286, 612285.71428571)]),
        "gori": Polygon([Point(-258542.85714286, 803857.14285714), Point(-258371.42857143, 805685.71428571),
                         Point(-264771.42857143, 808828.57142857), Point(-262342.85714286, 814571.42857143),
                         Point(-271342.85714286, 823057.14285714), Point(-282085.71428571, 837771.42857142),
                         Point(-283885.71428571, 853599.99999999), Point(-303228.57142857, 860571.42857142),
                         Point(-297314.28571429, 836971.42857142), Point(-291142.85714286, 819942.85714285),
                         Point(-292200, 805999.99999999), Point(-292142.85714286, 794714.28571428),
                         Point(-298371.42857143, 772085.71428571), Point(-288257.14285715, 772657.14285713),
                         Point(-277028.57142857, 785628.57142856), Point(-276685.71428572, 797399.99999999),
                         Point(-263771.42857143, 804914.28571428)])
    }

    air_zones = {
        "abkhazia": Polygon([Point(-187092.85714285, 460857.14285714), Point(-149378.57142856, 476285.71428571),
                             Point(-147664.28571428, 520000), Point(-175378.57142856, 599714.28571429),
                             Point(-174521.42857142, 644000), Point(-199664.28571428, 624000),
                             Point(-233949.99999999, 632857.14285714), Point(-271092.85714285, 596000)]),
        "nato": Polygon([Point(-283950, 568092.85714286), Point(-267949.99999999, 608950),
                         Point(-235092.85714285, 638950), Point(-208521.42857142, 672950),
                         Point(-225949.99999999, 738950), Point(-242521.42857142, 820950),
                         Point(-223092.85714285, 908857.14285714), Point(-283664.28571428, 953428.57142858),
                         Point(-389092.85714285, 904571.42857143), Point(-385092.85714285, 599714.28571429)]),
        "russia_west": Polygon([Point(-177092.85714285, 463428.57142857), Point(-33664.285714277, 224571.42857143),
                                Point(24621.428571437, 239714.28571429), Point(27192.857142866, 495714.28571429),
                                Point(-152235.7142857, 538571.42857143), Point(-146521.42857142, 474285.71428572)]),
        "russia_center": Polygon([Point(-154807.14285713, 539428.57142858), Point(-19664.285714275, 541714.28571429),
                                  Point(-20521.428571418, 688571.42857144), Point(-173949.99999999, 688857.14285715)]),
        "russia_east": Polygon([Point(-169949.99999999, 693714.28571429), Point(-24235.714285703, 694285.71428571),
                                Point(-94807.142857132, 945714.28571429), Point(-209949.99999999, 932571.42857143),
                                Point(-225092.85714285, 801142.85714286)])
    }

    red_force = {
        "Armor": {
            "Tank Platoon": (Russia.name, [
                Russia.Vehicle.Armor.MBT_T_90,
                Russia.Vehicle.Armor.MBT_T_90,
                Russia.Vehicle.Armor.MBT_T_90,
                Russia.Vehicle.Armor.MBT_T_90
            ]),
            "Tank Platoon 2": (Russia.name, [
                Russia.Vehicle.Armor.MBT_T_90,
                Russia.Vehicle.Armor.MBT_T_90,
                Russia.Vehicle.Armor.MBT_T_90,
                Russia.Vehicle.Armor.MBT_T_90
            ])
        },
        "LightArmor": {
            "Light Armor 01": (Russia.name, [
                Russia.Vehicle.Armor.IFV_BMP_3,
                Russia.Vehicle.Armor.IFV_BMP_3,
                Russia.Vehicle.Armor.IFV_BMP_3
            ]),
            "Light Armor 02": (Russia.name, [
                Russia.Vehicle.Armor.IFV_BMP_3,
                Russia.Vehicle.Armor.IFV_BMP_3,
                Russia.Vehicle.Armor.IFV_BMP_3
            ]),
            "Light Armor 03": (Russia.name, [
                Russia.Vehicle.Armor.IFV_BMP_3,
                Russia.Vehicle.Armor.IFV_BMP_3,
                Russia.Vehicle.Armor.IFV_BMP_3
            ])
        },
        "Artillery": {
            "Artillery": (Russia.name, [
                Russia.Vehicle.Artillery.SPH_2S1_Gvozdika,
                Russia.Vehicle.Artillery.SPH_2S1_Gvozdika,
                Russia.Vehicle.Artillery.SPH_2S1_Gvozdika,
                Russia.Vehicle.Artillery.SPH_2S1_Gvozdika,
                Russia.Vehicle.Unarmed.Transport_Ural_4320T,
                Russia.Vehicle.AirDefence.SAM_SA_19_Tunguska_2S6
            ])
        },
        "AirDefence": {
            "Air Defense": (Russia.name, [
                Russia.Vehicle.AirDefence.SAM_SA_6_Kub_LN_2P25,
                Russia.Vehicle.AirDefence.SAM_SA_6_Kub_STR_9S91,
                Russia.Vehicle.AirDefence.SAM_SA_6_Kub_LN_2P25,
                Russia.Vehicle.AirDefence.SAM_SA_6_Kub_STR_9S91,
                Russia.Vehicle.Unarmed.CP_SKP_11_ATC_Mobile_Command_Post
            ]),
            "Air Defense 02": (Russia.name, [
                Russia.Vehicle.AirDefence.SAM_SA_6_Kub_LN_2P25,
                Russia.Vehicle.AirDefence.SAM_SA_6_Kub_STR_9S91,
                Russia.Vehicle.AirDefence.SAM_SA_6_Kub_LN_2P25,
                Russia.Vehicle.AirDefence.SAM_SA_6_Kub_STR_9S91,
                Russia.Vehicle.Unarmed.CP_SKP_11_ATC_Mobile_Command_Post
            ])
        },
        "Supply": {
            "Russia Supply": (Russia.name, [
                Russia.Vehicle.Unarmed.Transport_GAZ_66,
                Russia.Vehicle.Unarmed.Fuel_Truck_ATZ_10,
                Russia.Vehicle.Unarmed.Transport_GAZ_3308,
                Russia.Vehicle.Unarmed.Transport_GAZ_3308,
                Russia.Vehicle.Unarmed.Transport_GAZ_3308
            ])
        }
    }

    blue_force = {
        "Armor": {
            "1st Tank Platoon 01": (USA.name, [
                USA.Vehicle.Armor.MBT_M1A2_Abrams,
                USA.Vehicle.Armor.MBT_M1A2_Abrams,
                USA.Vehicle.Armor.MBT_M1A2_Abrams,
                USA.Vehicle.Armor.MBT_M1A2_Abrams
            ])
        },
        "LightArmor": {
            "1st Light Armor Brigade 01": (USA.name, [
                USA.Vehicle.Armor.IFV_M2A2_Bradley,
                USA.Vehicle.Armor.IFV_M2A2_Bradley,
                USA.Vehicle.Armor.IFV_M2A2_Bradley,
            ])
        },
        "Artillery": {
            "1st Artillery Corps 01": (USA.name, [
                USA.Vehicle.Artillery.SPH_M109_Paladin,
                USA.Vehicle.Artillery.SPH_M109_Paladin,
                USA.Vehicle.Artillery.SPH_M109_Paladin,
                USA.Vehicle.Artillery.SPH_M109_Paladin,
                USA.Vehicle.Unarmed.HEMTT_TFFT,
                USA.Vehicle.AirDefence.SAM_Stinger_comm,
                USA.Vehicle.AirDefence.SAM_Stinger_MANPADS
            ])
        },
        "AirDefence": {
            "Air Defense Battery 01": (USA.name, [
                USA.Vehicle.Unarmed.APC_M1025_HMMWV,
                USA.Vehicle.Unarmed.APC_M1025_HMMWV,
                USA.Vehicle.AirDefence.SAM_Avenger_M1097,
                USA.Vehicle.AirDefence.SAM_Avenger_M1097,
                USA.Vehicle.AirDefence.SAM_Avenger_M1097,
                USA.Vehicle.AirDefence.SAM_Avenger_M1097
            ])
        },
        "Supply": {
            "US Supply": (USA.name, [
                USA.Vehicle.Unarmed.Transport_M818,
                USA.Vehicle.Unarmed.Transport_M818,
                USA.Vehicle.Unarmed.Tanker_M978_HEMTT,
                USA.Vehicle.Unarmed.HEMTT_TFFT
            ])
        }
    }

    def __init__(self, hide=True):
        self.hide = hide
        self.m = dcs.Mission()

        self.m.coalition["red"].swap_country(self.m.coalition["blue"], dcs.countries.Ukraine.name)

        self.red_airports = []  # type: List[dcs.terrain.Airport]
        self.blue_airports = []  # type: List[dcs.terrain.Airport]
        self.setup_airports()

    def dynamic_weather(self, baricsystem: str):
        if baricsystem == "dyncyclone":
            self.m.weather.dynamic_weather(dcs.weather.Weather.BaricSystem.Cyclone, 2)
        elif baricsystem == "dynanti":
            self.m.weather.dynamic_weather(dcs.weather.Weather.BaricSystem.AntiCyclone, 2)
        elif baricsystem == "dynnone":
            self.m.weather.dynamic_weather(dcs.weather.Weather.BaricSystem.None_, 2)
        else:
            self.m.random_weather = True

    def daytime(self, period):
        self.m.random_date()
        self.m.random_daytime(period)

    def setup_airports(self):
        caucasus = self.m.terrain  # type: dcs.terrain.Caucasus
        self.red_airports = caucasus.default_red_airports()
        for a in self.red_airports:
            self.setup_airport(a, "red", [dcs.vehicles.AirDefence.SAM_SA_19_Tunguska_2S6,
                                          dcs.vehicles.AirDefence.SPAAA_ZSU_23_4_Shilka])

        self.blue_airports = caucasus.default_blue_airports()
        for a in self.blue_airports:
            self.setup_airport(a, "blue", [dcs.vehicles.AirDefence.SAM_Avenger_M1097,
                                                dcs.vehicles.AirDefence.AAA_Vulcan_M163])

    def setup_airport(self, airport: dcs.terrain.Airport, side: str, air_def_units: List[dcs.unittype.VehicleType]):
        airport.set_coalition(side)

        if not airport.civilian:
            if airport.is_red():
                vg = dcs.templates.VehicleTemplate.Russia.sa10_site(self.m, airport.random_unit_zone().center(), 180)
                vg.hidden = self.hide
            elif airport.is_blue():
                dcs.templates.VehicleTemplate.USA.patriot_site(self.m, airport.random_unit_zone().center(), 330)
        else:
            slots = len(airport.parking_slots)
            airdef = int(round(random.random() + slots/20, 0))
            if airdef:
                vg = self.m.vehicle_group(
                    self.m.country("Russia") if airport.is_red() else self.m.country("USA"),
                    airport.name + " Air Defense",
                    air_def_units[random.randrange(0, len(air_def_units))],
                    airport.random_unit_zone().random_point(), 180)
                if airport.is_red():
                    vg.hidden = self.hide

                for i in range(1, airdef):
                    _type = air_def_units[random.randrange(0, len(air_def_units))]
                    u = self.m.vehicle(airport.name + " Air Defense #" + str(i), _type)
                    vg.add_unit(u)
                vg.formation_scattered(random.randrange(0, 359))

        return airport

    def add_civil_airtraffic(self, planes=(10, 20), helicopters=(0, 10), hidden=True):
        p_count = random.randrange(planes[0], planes[1])
        h_count = random.randrange(helicopters[0], helicopters[1])

        c_count = 1

        def civil_flight(countries, airports):
            country_str = countries[random.randrange(0, len(countries))]
            country = self.m.country(country_str)

            airport = airports[random.randrange(0, len(airports))]

            transports = [x for x in country.planes if x.task_default == dcs.task.Transport]
            ptype = random.choice(transports)

            slots = airport.free_parking_slot(ptype)

            x = random.randrange(dcs.terrain.Caucasus.bounds.bottom+100*1000, dcs.terrain.Caucasus.bounds.top-100*1000)
            y = random.randrange(dcs.terrain.Caucasus.bounds.left+600*1000, dcs.terrain.Caucasus.bounds.right-130*1000)

            pos = dcs.mapping.Point(x, y)
            name = "Civil " + str(c_count)
            rand = random.random()
            if 0.3 < rand < 0.5 and slots:
                pg = self.m.flight_group_from_airport(
                    country, name, ptype, airport,
                    start_type=random.choice([dcs.mission.StartType.Cold, dcs.mission.StartType.Warm]))
                pg.add_runway_waypoint(airport, distance=random.randrange(6000, 8000, 100))
                pg.add_waypoint(pos, random.randrange(5000, 8000, 100), 400)
            elif 0.5 < rand and slots:
                pg = self.m.flight_group_from_airport(country, name, ptype, airport)
                pg.uncontrolled = True
            else:
                bound = dcs.mapping.Rectangle(dcs.terrain.Caucasus.bounds.top-100*1000,
                                              dcs.terrain.Caucasus.bounds.left+200*1000,
                                              dcs.terrain.Caucasus.bounds.bottom+100*1000,
                                              dcs.terrain.Caucasus.bounds.right-130*1000)
                point = bound.random_point()

                pg = self.m.flight_group_inflight(
                    country, name, ptype, point, random.randrange(5000, 8000, 100), 400
                )
                tmp = pg.add_waypoint(dcs.mapping.Point(0, 0), pg.points[0].alt)
                wp = pg.add_runway_waypoint(airport, distance=random.randrange(6000, 8000, 100))
                heading = wp.position.heading_between_point(point)
                tmp.position = wp.position.point_from_heading(heading, 30*1000)
                pg.land_at(airport)
            pg.set_frequency(240)
            return pg

        def heli_transport_flight(countries, airports: List[dcs.terrain.Airport]):
            country_str = countries[random.randrange(0, len(countries))]
            country = self.m.country(country_str)

            transports = [x for x in country.helicopters
                          if x.task_default == dcs.task.Transport]
            htype = random.choice(transports)

            start_airport = random.choice(airports)
            rand = random.random()
            name = "Helicopter Transport " + str(c_count)
            if 0.7 < rand:
                bound = dcs.mapping.Rectangle.from_point(start_airport.position, 100*1000)
                pos = bound.random_point()
                hg = self.m.flight_group_inflight(country, name, htype, pos, random.randrange(800, 1500, 100), 200)
                hg.add_runway_waypoint(start_airport)
                hg.land_at(start_airport)
            elif 0.4 < rand < 0.7:
                hg = self.m.flight_group_from_airport(country, name, htype, start_airport)
                hg.uncontrolled = True
            else:
                dest_airport = None
                while True:
                    dest_airport = airports[random.randrange(0, len(airports))]
                    if dest_airport != start_airport:
                        break

                hg = self.m.flight_group_from_airport(
                    country, name, htype, start_airport, start_type=random.choice(list(dcs.mission.StartType))
                )
                hg.add_runway_waypoint(start_airport)
                hg.add_runway_waypoint(dest_airport)
                hg.land_at(dest_airport)
            return hg

        # red
        red_countries = [dcs.countries.Russia.name]
        for i in range(0, int(p_count / 2)):
            cf = civil_flight(red_countries, self.red_airports)
            cf.hidden = hidden
            cf.points[0].tasks.append(dcs.task.SetInvisibleCommand())
            c_count += 1

        for i in range(0, int(h_count / 2)):
            hf = heli_transport_flight(red_countries, self.red_airports)
            hf.hidden = hidden
            hf.points[0].tasks.append(dcs.task.SetInvisibleCommand())
            c_count += 1

        # blue
        blue_countries = [dcs.countries.USA.name, dcs.countries.Ukraine.name, dcs.countries.Georgia.name]
        for i in range(0, int(p_count / 2)):
            cf = civil_flight(blue_countries, self.blue_airports)
            cf.hidden = hidden
            cf.points[0].tasks.append(dcs.task.SetInvisibleCommand())
            c_count += 1

        for i in range(0, int(h_count / 2)):
            hf = heli_transport_flight(blue_countries, self.blue_airports)
            hf.hidden = hidden
            hf.points[0].tasks.append(dcs.task.SetInvisibleCommand())
            c_count += 1

    def add_uncontrolled_military_planes(self, airports: List[dcs.terrain.Airport],
                                         planes: List[Tuple[str, dcs.unittype.FlyingType, int]], hidden=True):

        g_idx = 1
        while planes:
            country, ptype, group_size = planes.pop()

            while True:
                airport = random.choice(airports)

                slots = airport.free_parking_slots(ptype)
                if len(slots) >= group_size:
                    break

            c = self.m.country(country)
            pg = self.m.flight_group_from_airport(c, ptype.id + " Flight #" + str(g_idx),
                                                  ptype, airport, parking_slots=slots, group_size=group_size)
            pg.uncontrolled = True
            pg.hidden = hidden
            g_idx += 1

    def place_players(self, start, aircraft_types: List[Tuple[str, str]],
                      airports: List[dcs.terrain.Airport],
                      group_size,
                      maintask,
                      placement_rect=None) -> List[dcs.unitgroup.FlyingGroup]:

        aircraft_groups = []  # type: List[dcs.unitgroup.FlyingGroup]
        name = "Airmaster "
        c = 1
        for _type in aircraft_types:
            country = self.m.country(_type[0])
            aircraft_type = _type[1]
            aircraft_type = dict(dcs.planes.plane_map, **dcs.helicopters.helicopter_map)[aircraft_type]

            airport = airports[random.randrange(0, len(airports))]
            if start == "inflight":
                rp = placement_rect.random_point() if placement_rect else dcs.mapping.Rectangle.from_point(
                        airport.position, 20*1000).random_point()
                altitude = random.randrange(300, 1000, 100) if aircraft_type.helicopter else random.randrange(2000, 5000, 100)
                pg = self.m.flight_group_inflight(
                    country, name + str(c), aircraft_type, rp, altitude,
                    maintask=maintask, group_size=group_size)
            else:
                pg = self.m.flight_group_from_airport(
                    country, name + str(c), aircraft_type, airport,
                    maintask=maintask, start_type=dcs.mission.StartType.from_string(start), group_size=group_size
                )
                pg.add_runway_waypoint(airport)
            aircraft_groups.append(pg)

            for u in pg.units:
                u.set_client()
            c += 1

        if group_size == 1:
            aircraft_groups[0].units[0].set_player()

        return aircraft_groups

    def save(self, filename, stats):
        self.m.save(filename)
        if stats:
            self.m.print_stats(self.m.stats())


class Refueling(BasicScenario):
    def __init__(self, aircraft_types: List[Tuple[str, str]], playercount: int, start: str, unhide):
        super(Refueling, self).__init__()

        self.add_civil_airtraffic(hidden=not unhide)

        caucasus = self.m.terrain  # type: dcs.terrain.Caucasus

        usa = self.m.country(dcs.countries.USA.name)
        ukraine = self.m.country(dcs.countries.Ukraine.name)

        batumi = caucasus.batumi()
        vaziani = caucasus.vaziani()

        kutaisi = caucasus.kutaisi()
        kobuleti = caucasus.kobuleti()
        blue_military = [kutaisi, kobuleti]

        planes = [
            (dcs.countries.USA.name, dcs.countries.USA.Plane.F_16C_bl_52d, 4),
            (dcs.countries.USA.name, dcs.countries.USA.Plane.F_15E, 2),
            (dcs.countries.USA.name, dcs.countries.USA.Plane.F_A_18C, 4),
            (dcs.countries.Georgia.name, dcs.countries.Georgia.Plane.Su_25T, 4)
        ]

        self.add_uncontrolled_military_planes(blue_military, planes, False)

        frequency = 140
        orbit_rect = dcs.mapping.Rectangle(
            int(batumi.position.x + 120*1000), int(batumi.position.y - 100 * 1000), int(batumi.position.x), int(vaziani.position.y))

        pos, heading, race_dist = Refueling.random_orbit(orbit_rect)
        awacs = self.m.awacs_flight(
            usa,
            "AWACS",
            plane_type=dcs.planes.E_3A,
            airport=None,
            position=pos,
            race_distance=race_dist, heading=heading,
            altitude=random.randrange(4000, 5500, 100), frequency=frequency)

        self.m.escort_flight(usa, "AWACS Escort", dcs.countries.USA.Plane.F_15E, None, awacs, group_size=2)

        pos, heading, race_dist = Refueling.random_orbit(orbit_rect)
        refuel_net = self.m.refuel_flight(
            ukraine,
            "Tanker IL",
            dcs.planes.IL_78M,
            airport=None,
            position=pos,
            race_distance=race_dist, heading=heading,
            altitude=random.randrange(4000, 5500, 100), speed=750, frequency=frequency)

        pos, heading, race_dist = Refueling.random_orbit(orbit_rect)
        refuel_rod = self.m.refuel_flight(
            usa,
            "Tanker KC",
            dcs.planes.KC_135,
            airport=None,
            position=pos,
            race_distance=race_dist, heading=heading,
            altitude=random.randrange(4000, 5500, 100), frequency=frequency)

        player_groups = self.place_players(start, aircraft_types, blue_military,
                                           placement_rect=orbit_rect,
                                           group_size=playercount, maintask=None)

        if start == "inflight":
            fuel_percent = 0.2
        else:
            fuel_percent = 0.3

        for pg in player_groups:
            airport = self.m.terrain.airport_by_id(pg.points[0].airdrome_id)
            if not airport:
                airport = random.choice(blue_military)

            for u in pg.units:
                u.fuel *= fuel_percent

            if pg.units[0].unit_type in [dcs.planes.A_10C]:
                pg.add_waypoint(refuel_rod.points[1].position, 4000)
            else:
                pg.add_waypoint(refuel_net.points[1].position, 4000)

            pg.add_runway_waypoint(airport)
            pg.land_at(airport)
            pg.load_loadout("Combat Air Patrol")

        goal = dcs.goals.Goal("home and alive")
        goal.rules.append(dcs.condition.UnitAlive(pg.units[0].id))
        tz = self.m.triggers.add_triggerzone(airport.position, 1000, name="home")
        goal.rules.append((dcs.condition.UnitInZone(pg.units[0].id, tz.id)))
        self.m.goals.add_offline(goal)

        self.m.set_sortie_text("Refuel your plane")
        self.m.set_description_text("""Random generated refueling test mission.
{count} {type} are/is prepared for a refueling training mission.""".format(
            count=playercount, type=", ".join([x[1] for x in aircraft_types])))
        self.m.set_description_bluetask_text("""Find your tanker and do a full refuel.
Afterwards land at your designated homebase.

AWACS and Tankers are reachable on {freq} Mhz VHF-AM.
KC-135 Tanker has TACAN 10X activated.""".format(freq=frequency))

    @staticmethod
    def random_orbit(rect: dcs.mapping.Rectangle):
        x1 = random.randrange(rect.bottom, rect.top)
        sy = rect.left
        y1 = random.randrange(sy, rect.right)
        heading = 90 if y1 < (sy + (rect.right - sy) / 2) else 270
        heading = random.randrange(heading - 20, heading + 20)
        race_dist = random.randrange(80*1000, 120*1000)
        return dcs.mapping.Point(x1, y1), heading, race_dist


class CAS(BasicScenario):
    air_force = {
        "blue": {
            "SEAD": {
                "SEAD 01": (USA.name, 2, dcs.planes.F_16C_bl_52d)
            },
            "CAP": {
                "Air Patrol": (USA.name, 2, dcs.planes.F_15C)
            },
            "CAS": {
                "AirCav": (USA.name, 2, dcs.helicopters.AH_64D)
            }
        },
        "red": {
            "CAS": {
                "Hindis": (Russia.name, 2, dcs.helicopters.Mi_24V)
            }
        }
    }

    nato_airspace = Polygon([
        Point(-272807.14285714, 563521.42857143), Point(-267949.99999999, 608950), Point(-235092.85714285, 638950),
        Point(-208521.42857142, 672950), Point(-225949.99999999, 738950), Point(-242521.42857142, 820950),
        Point(-243092.85714285, 912857.14285714), Point(-273949.99999999, 963714.28571429),
        Point(-415664.28571428, 963714.28571429), Point(-392807.14285714, 569142.85714286)])

    def __init__(self, aircraft_types: List[Tuple[str, str]], playercount: int, start: str, unhide):
        super(CAS, self).__init__(not unhide)

        caucasus = self.m.terrain  # type: dcs.terrain.Caucasus

        kutaisi = caucasus.kutaisi()
        kobuleti = caucasus.kobuleti()
        blue_military_airport = [kutaisi, kobuleti]

        red_military_airport = [caucasus.sochi()]

        battle_point = BasicScenario.battle_zones["zugidi"].random_point()

        front_hdg = 30
        isred = False
        for force in [self.blue_force, self.red_force]:
            attack_hdg = (300 + (180 if isred else 0)) % 360
            defense_hdg = (attack_hdg + 180) % 360
            rp = battle_point.point_from_heading(defense_hdg, 7*1000)
            for force_type in force:
                for conf in force[force_type]:
                    dist_hdg = front_hdg if random.getrandbits(1) else front_hdg + 180
                    rp = rp.point_from_heading(dist_hdg, random.randrange(100, 1000))
                    if force_type in ["Artillery", "Supply"]:
                        rp = rp.point_from_heading(random.randrange(defense_hdg-30, defense_hdg+30), random.randrange(500, 3000))
                    else:
                        rp = rp.point_from_heading(random.randrange(attack_hdg-30, attack_hdg+30), random.randrange(0, 500))

                    plat = force[force_type][conf]
                    c = self.m.country(plat[0])
                    g = self.m.vehicle_group_platoon(c, conf, plat[1], rp)
                    g.formation_scattered(attack_hdg)
                    if isred:
                        g.hidden = self.hide

                    if force_type not in ["Artillery", "Supply"] and isred:
                        wp = g.add_waypoint(g.position.point_from_heading(attack_hdg, 12000))
                        wp.action = dcs.point.PointAction.Vee
                        g.add_waypoint(wp.position.point_from_heading(random.randrange(attack_hdg-10, attack_hdg+30), 5000))

                    if force_type in ["Artillery"]:
                        for x in range(0, 10):
                            rect = dcs.mapping.Rectangle.from_point(battle_point.point_from_heading(attack_hdg, 7000), 2000)
                            g.points[0].tasks.append(dcs.task.FireAtPoint(rect.random_point(), 30, 30))
            isred = True

        # rp = dcs.mapping.Rectangle.from_point(rp, 200).random_point()
        # fac = self.m.vehicle_group(usa, "FAC", USA.Vehicle.Armor.APC_M1126_Stryker_ICV, rp)
        # fac.points[0].tasks.append(dcs.task.EPLRS(1))
        # fac.points[0].tasks.append(dcs.task.FAC())
        # fac.points[0].tasks.append(dcs.task.SetInvisibleCommand())
        # fac.points[0].tasks.append(dcs.task.OptROE(dcs.task.OptROE.Values.WeaponHold))

        for air_force_idx in self.air_force:
            air_force = self.air_force[air_force_idx]
            airports = blue_military_airport if air_force_idx == "blue" else red_military_airport
            for force_type in air_force:
                for conf in air_force[force_type]:
                    conf_data = air_force[force_type][conf]
                    airport = random.choice(airports)
                    country = self.m.country(conf_data[0])
                    if force_type == "CAP":
                        nato_rect = self.nato_airspace.outbound_rectangle()
                        pp1, pp2 = nato_rect.random_distant_points(nato_rect.width() * 0.6)
                        self.m.patrol_flight(country, conf, conf_data[2], airport,
                                             pp1, pp2,
                                             group_size=conf_data[1])
                    else:
                        try:
                            pg = self.m.flight_group_from_airport(
                                country, conf, conf_data[2], airport, dcs.task.MainTask.map[force_type],
                                dcs.mission.StartType.Runway, group_size=conf_data[1]
                            )
                        except dcs.terrain.RunwayOccupiedError:
                            pg = self.m.flight_group_from_airport(
                                country, conf, conf_data[2], airport, dcs.task.MainTask.map[force_type],
                                dcs.mission.StartType.Warm, group_size=conf_data[1]
                            )
                        pg.add_runway_waypoint(airport)
                        if air_force_idx == "red":
                            pg.hidden = self.hide

                        if force_type in ["SEAD", "CAS"]:
                            pg.add_waypoint(battle_point, 1000)

        player_groups = self.place_players(start, aircraft_types, blue_military_airport, playercount, dcs.task.CAS)
        for pg in player_groups:
            pg.add_waypoint(battle_point, 0)

        self.m.set_sortie_text("Random CAS")
        self.m.set_description_text("""Random generated CAS test mission.""")
        self.m.set_description_bluetask_text(
"""Support your ground troops at waypoint 2.
Take care of enemy air defense.

2 F-16 are supporting you with a SEAD strike""")


class CAP(BasicScenario):
    air_force = {
        "red": {
            "CAS": {
                "Hindis": {
                    "country": Russia.name,
                    "size": 2,
                    "rating": 10,
                    "type": dcs.helicopters.Mi_24V,
                    "loadout": "4x9M114, 80xS-8"
                },
                "Blackies": {
                    "country": Russia.name,
                    "size": 2,
                    "rating": 10,
                    "type": Russia.Helicopter.Ka_50,
                    "loadout": None
                }
            },
            "CAP": {
                "Mig 21": {
                    "country": Russia.name,
                    "size": 2,
                    "rating": 40,
                    "type": dcs.planes.MiG_21Bis,
                    "loadout": None
                },
                "Mig 15": {
                    "country": Russia.name,
                    "size": 2,
                    "rating": 20,
                    "type": dcs.planes.MiG_15bis,
                    "loadout": None
                },
                "Mig 29": {
                    "country": Russia.name,
                    "size": 2,
                    "rating": 70,
                    "type": dcs.planes.MiG_29A,
                    "loadout": None
                },
                "Su 33": {
                    "country": Russia.name,
                    "size": 2,
                    "rating": 80,
                    "type": dcs.planes.Su_33,
                    "loadout": "R-73*2,R-27ET*2,R-27ER*6,ECM"
                }
            }
        }
    }

    def __init__(self, aircraft_types: List[Tuple[str, str]], playercount: int, start: str, unhide=False):
        super(CAP, self).__init__(not unhide)

        caucasus = self.m.terrain
        blue_military_airport = [caucasus.kobuleti(), caucasus.kutaisi()]

        player_groups = self.place_players(start, aircraft_types, blue_military_airport, playercount, dcs.task.CAP)
        patrol_zone = self.air_zones["nato"].outbound_rectangle().resize(0.7)
        p1, p2 = patrol_zone.random_distant_points(patrol_zone.width() * 0.6)
        for pg in player_groups:
            pg.load_loadout("Combat Air Patrol")
            pg.add_waypoint(p1, 5000)
            pg.add_waypoint(p2, 5000)

        vhf_am = 140

        usa = self.m.country("USA")
        ukraine = self.m.country(dcs.countries.Ukraine.name)
        race_dist = random.randrange(80*1000, 120*1000, 1000)
        p1, p2 = patrol_zone.random_distant_points(race_dist)
        p1.heading_between_point(p2)
        awacs = self.m.awacs_flight(
            usa,
            "AWACS",
            plane_type=dcs.planes.E_3A,
            airport=None,
            position=p1,
            race_distance=race_dist, heading=p1.heading_between_point(p2),
            altitude=random.randrange(4000, 5500, 100), frequency=vhf_am)

        self.m.escort_flight(usa, "AWACS Escort", dcs.countries.USA.Plane.F_15E, None, awacs, 2)

        race_dist = random.randrange(80*1000, 120*1000, 1000)
        p1, p2 = patrol_zone.random_distant_points(race_dist)
        p1.heading_between_point(p2)
        self.m.refuel_flight(
            ukraine,
            "Tanker IL",
            dcs.planes.IL_78M,
            airport=None,
            position=p1,
            race_distance=race_dist, heading=p1.heading_between_point(p2),
            altitude=random.randrange(4000, 5500, 100), speed=750, frequency=vhf_am)

        i = 0
        for zone in ["russia_east", "russia_west"]:
            rcaps = list(self.air_force["red"]["CAP"].keys())
            rcap = self.air_force["red"]["CAP"][random.choice(rcaps)]
            cap_country = self.m.country(rcap["country"])
            p1, p2 = self.air_zones[zone].outbound_rectangle().resize(0.6).random_distant_points(80*1000)
            pf = self.m.patrol_flight(cap_country, cap_country.name + " CAP " + str(i), rcap["type"], None,
                                      p1, p2, group_size=rcap["size"])
            pf.hidden = not unhide
            if rcap["loadout"]:
                pf.load_loadout(rcap["loadout"])
            i += 1

        for x in range(0, random.randrange(1, 3)):
        #for attack_type in self.air_force["red"].keys():
            attack_type = random.choice(list(self.air_force["red"].keys()))
            af = self.air_force["red"][attack_type][random.choice(list(self.air_force["red"][attack_type].keys()))]
            spawn_rect = self.air_zones["russia_east"].outbound_rectangle()
            spawn_rect = dcs.mapping.Rectangle(spawn_rect.bottom + 40000, spawn_rect.left,
                                               spawn_rect.bottom, spawn_rect.right)
            att_country = self.m.country(af["country"])
            start_airport = caucasus.sochi() if af["type"].helicopter else random.choice(self.red_airports)
            attack_airport = random.choice(self.blue_airports)

            pos = attack_airport.position.point_from_heading(
                random.randrange(290, 410), 60*1000) if af["type"].helicopter else spawn_rect.random_point()
            attack_flight = self.m.flight_group(
                att_country,
                att_country.name + " " + attack_type,
                af["type"], None if start == "inflight" else start_airport, pos, random.randrange(2000, 4000, 100),
                maintask=dcs.task.MainTask.map[attack_type], group_size=af["size"])
            attack_flight.hidden = not unhide
            if attack_flight.starts_from_airport():
                attack_flight.add_runway_waypoint(start_airport)

            if af["loadout"]:
                attack_flight.load_loadout(af["loadout"])
            if attack_type == "CAS":
                wp = attack_flight.add_waypoint(attack_airport.position, 2000)
                wp.tasks.append(dcs.task.EngageTargetsInZone(attack_airport.position))
            else:
                attack_flight.add_waypoint(awacs.position, 6000)

            attack_flight.add_runway_waypoint(start_airport)
            attack_flight.land_at(start_airport)

        self.m.set_sortie_text("Random CAP")
        self.m.set_description_text("""Random generated CAP test mission.""")
        self.m.set_description_bluetask_text(
"""Protect your AWACS and Tanker

AWACS and tanker are reachable on VHF-AM {freq} Mhz.""".format(freq=vhf_am))


def main():

    types = [
        (dcs.countries.USA.name, dcs.planes.A_10C.id),
        (dcs.countries.Georgia.name, dcs.planes.Su_25T.id),
        (dcs.countries.USA.name, dcs.planes.M_2000C.id),
        (dcs.countries.USA.name, dcs.helicopters.Ka_50.id),
        (dcs.countries.USA.name, USA.Plane.MiG_21Bis.id)
    ]
    aircraft_types = [x[1] for x in types]
    parser = argparse.ArgumentParser(description="Random DCS mission generator")

    parser.add_argument("-a", "--aircrafttype", default=dcs.planes.Su_25T.id,
                        choices=aircraft_types,
                        help="Player aircraft type")
    parser.add_argument("-p", "--playercount", default=1, type=int)
    parser.add_argument("-s", "--start", default="inflight", choices=["inflight", "runway", "warm", "cold"])
    parser.add_argument("-t", "--missiontype", default="main", choices=["main", "CAS", "CAP", "refuel"])
    parser.add_argument("-d", "--daytime", choices=["random", "day", "night", "dusk", "dawn", "noon"], default="random")
    parser.add_argument("-w", "--weather", choices=["dynamic", "dyncyclone", "dynanti", "dynone", "clear"], default="dynamic")
    parser.add_argument("-u", "--unhide", action="store_true", default=False, help="Show enemy pre mission")
    parser.add_argument("--show-stats", action="store_true", default=False, help="Show generated missions stats")
    parser.add_argument("-o", "--output", help="Name and path of the generated mission",
                        default=os.path.join(os.path.expanduser("~"),"Saved Games\\DCS\\Missions\\random.miz"))

    args = parser.parse_args()
    missiontype = args.missiontype
    if args.aircrafttype in [
        dcs.planes.Su_25T.id,
        dcs.helicopters.Ka_50.id,
        dcs.planes.A_10C.id] and args.missiontype == "main":
        missiontype = "CAS"
    if args.aircrafttype in [dcs.planes.M_2000C.id, dcs.planes.MiG_21Bis.id] and args.missiontype == "main":
        missiontype = "CAP"
    if args.aircrafttype == dcs.planes.A_10C.id:
        if args.missiontype not in ["CAS", "refuel"]:
            missiontype = "CAS"

    if missiontype == "refuel":
        supported = [dcs.planes.A_10C.id, dcs.planes.M_2000C.id]
        types = [x for x in types if x[1] in supported] if args.playercount > 1 else [x for x in types if x[1] == args.aircrafttype]
        s = Refueling(types, args.playercount, args.start, args.unhide)
    elif missiontype == "CAS":
        types = types if args.playercount > 1 else [x for x in types if x[1] == args.aircrafttype]
        s = CAS(types, args.playercount, args.start, args.unhide)
    elif missiontype == "CAP":
        supported = [dcs.planes.M_2000C.id, dcs.planes.MiG_21Bis]
        types = [x for x in types if x[1] in supported] if args.playercount > 1 else [x for x in types if x[1] == args.aircrafttype]
        s = CAP(types, args.playercount, args.start, args.unhide)
    else:
        raise NotImplementedError(missiontype + " not implemented")

    s.daytime(args.daytime)
    if args.weather == "dynamic":
        s.dynamic_weather(args.weather)
    s.save(args.output, args.show_stats)

    print("Mission created: " + args.output)
    return 0


if __name__ == '__main__':
    sys.exit(main())



# This file is generated from pydcs_export.lua

from .weapons_data import Weapons
from . import task
from .unittype import FlyingType
from enum import Enum


class HelicopterType(FlyingType):
    helicopter = True


class Ka_50(HelicopterType):
    id = "Ka-50"
    flyable = True
    height = 5.6
    width = 14.4
    length = 15.84
    fuel_max = 1450
    max_speed = 300
    chaff = 0
    flare = 128
    charge_total = 128
    chaff_charge_size = 0
    flare_charge_size = 1
    radio_frequency = 124

    panel_radio = {
        1: {
            "channels": {
                7: 40,
                1: 21.5,
                2: 25.7,
                4: 28,
                8: 50,
                9: 55.5,
                5: 30,
                10: 59.9,
                3: 27,
                6: 32
            },
        },
        2: {
            "channels": {
                15: 0.995,
                13: 0.583,
                7: 0.443,
                1: 0.625,
                2: 0.303,
                4: 0.591,
                8: 0.215,
                16: 1.21,
                9: 0.525,
                5: 0.408,
                10: 1.065,
                14: 0.283,
                3: 0.289,
                6: 0.803,
                12: 0.35,
                11: 0.718
            },
        },
    }

    class Liveries:

        class Georgia(Enum):
            georgia_camo = "georgia camo"

        class Australia(Enum):
            Russia_Worn_Black = "Russia Worn Black"

        class Germany(Enum):
            german_8320 = "german 8320"
            german_8332 = "german 8332"

        class Israel(Enum):
            Israel_IAF_camo_1 = "Israel IAF camo 1"
            Israel_IAF_camo_2 = "Israel IAF camo 2"
            Israel_IAF_camo_3 = "Israel IAF camo 3"

        class Norway(Enum):
            norway_camo = "norway camo"

        class Spain(Enum):
            Spain_SAA_Arido = "Spain SAA Arido"
            Spain_SAA_Boscoso = "Spain SAA Boscoso"
            Spain_SAA_Standard = "Spain SAA Standard"

        class Ukraine(Enum):
            Ukraine_Demo = "Ukraine Demo"
            ukraine_camo_1 = "ukraine camo 1"
            ukraine_camo_1_dirt = "ukraine camo 1 dirt"

        class Belgium(Enum):
            belgium_sar = "belgium sar"
            belgium_camo = "belgium camo"
            belgium_olive = "belgium olive"

        class Greece(Enum):
            Greek_Army_Aviation = "Greek Army Aviation"
            Hellenic_Navy_Aviation = "Hellenic Navy Aviation"
            Hellenic_Navy_Aviation_2 = "Hellenic Navy Aviation 2"

        class UK(Enum):
            uk_camo = "uk camo"

        class France(Enum):
            France_Armee_de_Terre_1 = "France Armee de Terre 1"
            France_Armee_de_Terre_2 = "France Armee de Terre 2"
            France_Armee_de_Terre_Desert = "France Armee de Terre Desert"

        class Abkhazia(Enum):
            Abkhazia_1 = "Abkhazia 1"

        class Russia(Enum):
            Russia_Standard_Army = "Russia Standard Army"
            Russia_DOSAAF = "Russia DOSAAF"
            Russia_Demo__024 = "Russia Demo #024"
            Russia_Demo__22__Black_Shark = "Russia Demo #22 `Black Shark`"
            Russia_Demo__Werewolf = "Russia Demo `Werewolf`"
            Russia_Fictional_Swedish = "Russia Fictional Swedish"
            Russia_fictional_desert_scheme = "Russia fictional desert scheme"
            Russia_Fictional_Olive_Grey = "Russia Fictional Olive Grey"
            Russia_Fictional_Snow_Splatter = "Russia Fictional Snow Splatter"
            Russia_Fictional_Tropic_Green = "Russia Fictional Tropic Green"
            Russia_New_Year = "Russia New Year"
            Russia_Standard_Army__Worn = "Russia Standard Army (Worn)"
            Russia_Worn_Black = "Russia Worn Black"

        class Italy(Enum):
            Italy_Aeronautica_Militare = "Italy Aeronautica Militare"
            Italy_Esercito_Italiano = "Italy Esercito Italiano"

        class SouthOssetia(Enum):
            South_Ossetia_1 = "South Ossetia 1"

        class USA(Enum):
            us_army = "us army"
            us_marines_1 = "us marines 1"
            us_marines_2 = "us marines 2"

        class Denmark(Enum):
            denmark_camo = "denmark camo"
            Denmark_navy_trainer = "Denmark navy trainer"

        class Canada(Enum):
            canadian_forces = "canadian forces"

        class TheNetherlands(Enum):
            Netherlands_RNAF = "Netherlands RNAF"
            Netherlands_RNAF_wooded = "Netherlands RNAF wooded"

        class Turkey(Enum):
            Turkey_Fictional_Light_Gray = "Turkey Fictional Light Gray"
            Turkey_Fictional_1 = "Turkey Fictional 1"
            Turkey_Fictional = "Turkey Fictional"
            Turkey_fictional_desert_scheme = "Turkey fictional desert scheme"

    class Pylon1:
        B_8V20A_CM = (1, Weapons.B_8V20A_CM)
        B_8V20A_OFP2 = (1, Weapons.B_8V20A_OFP2)
        B_8V20A_OM = (1, Weapons.B_8V20A_OM)
        APU_6___6_9A4172_Vikhr = (1, Weapons.APU_6___6_9A4172_Vikhr)
        Kh_25ML = (1, Weapons.Kh_25ML)
        B_8V20A___20_S_8KOM = (1, Weapons.B_8V20A___20_S_8KOM)
        B_13L___5_S_13_OF = (1, Weapons.B_13L___5_S_13_OF)
        UPK_23_250 = (1, Weapons.UPK_23_250)
        KMGU_2___96_AO_2_5RT = (1, Weapons.KMGU_2___96_AO_2_5RT)
        KMGU_2___96_PTAB_2_5KO = (1, Weapons.KMGU_2___96_PTAB_2_5KO)
        FAB_250 = (1, Weapons.FAB_250)
        FAB_500_M62 = (1, Weapons.FAB_500_M62)
        Fuel_tank_Ka_50 = (1, Weapons.Fuel_tank_Ka_50)

    class Pylon2:
        B_8V20A_CM = (2, Weapons.B_8V20A_CM)
        B_8V20A_OFP2 = (2, Weapons.B_8V20A_OFP2)
        B_8V20A_OM = (2, Weapons.B_8V20A_OM)
        B_8V20A___20_S_8KOM = (2, Weapons.B_8V20A___20_S_8KOM)
        B_13L___5_S_13_OF = (2, Weapons.B_13L___5_S_13_OF)
        UPK_23_250 = (2, Weapons.UPK_23_250)
        KMGU_2___96_AO_2_5RT = (2, Weapons.KMGU_2___96_AO_2_5RT)
        KMGU_2___96_PTAB_2_5KO = (2, Weapons.KMGU_2___96_PTAB_2_5KO)
        FAB_250 = (2, Weapons.FAB_250)
        FAB_500_M62 = (2, Weapons.FAB_500_M62)
        Fuel_tank_Ka_50 = (2, Weapons.Fuel_tank_Ka_50)

    class Pylon3:
        B_8V20A_CM = (3, Weapons.B_8V20A_CM)
        B_8V20A_OFP2 = (3, Weapons.B_8V20A_OFP2)
        B_8V20A_OM = (3, Weapons.B_8V20A_OM)
        B_8V20A___20_S_8KOM = (3, Weapons.B_8V20A___20_S_8KOM)
        B_13L___5_S_13_OF = (3, Weapons.B_13L___5_S_13_OF)
        UPK_23_250 = (3, Weapons.UPK_23_250)
        KMGU_2___96_AO_2_5RT = (3, Weapons.KMGU_2___96_AO_2_5RT)
        KMGU_2___96_PTAB_2_5KO = (3, Weapons.KMGU_2___96_PTAB_2_5KO)
        FAB_250 = (3, Weapons.FAB_250)
        FAB_500_M62 = (3, Weapons.FAB_500_M62)
        Fuel_tank_Ka_50 = (3, Weapons.Fuel_tank_Ka_50)

    class Pylon4:
        B_8V20A_CM = (4, Weapons.B_8V20A_CM)
        B_8V20A_OFP2 = (4, Weapons.B_8V20A_OFP2)
        B_8V20A_OM = (4, Weapons.B_8V20A_OM)
        APU_6___6_9A4172_Vikhr = (4, Weapons.APU_6___6_9A4172_Vikhr)
        Kh_25ML = (4, Weapons.Kh_25ML)
        B_8V20A___20_S_8KOM = (4, Weapons.B_8V20A___20_S_8KOM)
        B_13L___5_S_13_OF = (4, Weapons.B_13L___5_S_13_OF)
        UPK_23_250 = (4, Weapons.UPK_23_250)
        KMGU_2___96_AO_2_5RT = (4, Weapons.KMGU_2___96_AO_2_5RT)
        KMGU_2___96_PTAB_2_5KO = (4, Weapons.KMGU_2___96_PTAB_2_5KO)
        FAB_250 = (4, Weapons.FAB_250)
        FAB_500_M62 = (4, Weapons.FAB_500_M62)
        Fuel_tank_Ka_50 = (4, Weapons.Fuel_tank_Ka_50)

    pylons = {1, 2, 3, 4}

    tasks = [task.CAS, task.GroundAttack, task.Escort, task.AFAC, task.AntishipStrike]
    task_default = task.CAS


class Ka_52(HelicopterType):
    id = "Ka-52"
    height = 5.6
    width = 14.4
    length = 15.84
    fuel_max = 1500
    max_speed = 300
    chaff = 0
    flare = 128
    charge_total = 128
    chaff_charge_size = 1
    flare_charge_size = 1

    class Liveries:

        class Georgia(Enum):
            standard = "standard"

        class Syria(Enum):
            standard = "standard"

        class Finland(Enum):
            standard = "standard"

        class Australia(Enum):
            standard = "standard"

        class Germany(Enum):
            standard = "standard"

        class SaudiArabia(Enum):
            standard = "standard"

        class Israel(Enum):
            standard = "standard"

        class Croatia(Enum):
            standard = "standard"

        class CzechRepublic(Enum):
            standard = "standard"

        class Norway(Enum):
            standard = "standard"

        class Romania(Enum):
            standard = "standard"

        class Spain(Enum):
            standard = "standard"

        class Ukraine(Enum):
            standard = "standard"

        class Belgium(Enum):
            standard = "standard"

        class Slovakia(Enum):
            standard = "standard"

        class Greece(Enum):
            standard = "standard"

        class UK(Enum):
            standard = "standard"

        class Insurgents(Enum):
            standard = "standard"

        class Hungary(Enum):
            standard = "standard"

        class France(Enum):
            standard = "standard"

        class Abkhazia(Enum):
            standard = "standard"

        class Russia(Enum):
            standard = "standard"

        class Sweden(Enum):
            standard = "standard"

        class Austria(Enum):
            standard = "standard"

        class Switzerland(Enum):
            standard = "standard"

        class Italy(Enum):
            standard = "standard"

        class SouthOssetia(Enum):
            standard = "standard"

        class SouthKorea(Enum):
            standard = "standard"

        class Iran(Enum):
            standard = "standard"

        class China(Enum):
            standard = "standard"

        class Pakistan(Enum):
            standard = "standard"

        class Belarus(Enum):
            standard = "standard"

        class NorthKorea(Enum):
            standard = "standard"

        class Iraq(Enum):
            standard = "standard"

        class Kazakhstan(Enum):
            standard = "standard"

        class Bulgaria(Enum):
            standard = "standard"

        class Serbia(Enum):
            standard = "standard"

        class India(Enum):
            standard = "standard"

        class USAFAggressors(Enum):
            standard = "standard"

        class USA(Enum):
            standard = "standard"

        class Denmark(Enum):
            standard = "standard"

        class Egypt(Enum):
            standard = "standard"

        class Canada(Enum):
            standard = "standard"

        class TheNetherlands(Enum):
            standard = "standard"

        class Turkey(Enum):
            standard = "standard"

        class Japan(Enum):
            standard = "standard"

        class Poland(Enum):
            standard = "standard"

    class Pylon1:
        B_8V20A_CM = (1, Weapons.B_8V20A_CM)
        B_8V20A_OFP2 = (1, Weapons.B_8V20A_OFP2)
        B_8V20A_OM = (1, Weapons.B_8V20A_OM)
        B_8V20A___20_S_8KOM = (1, Weapons.B_8V20A___20_S_8KOM)
        R_73 = (1, Weapons.R_73)
        APU_6___6_9A4172_Vikhr = (1, Weapons.APU_6___6_9A4172_Vikhr)
        B_13L___5_S_13_OF = (1, Weapons.B_13L___5_S_13_OF)
        KMGU_2___96_AO_2_5RT = (1, Weapons.KMGU_2___96_AO_2_5RT)
        KMGU_2___96_PTAB_2_5KO = (1, Weapons.KMGU_2___96_PTAB_2_5KO)
        FAB_250 = (1, Weapons.FAB_250)

    class Pylon2:
        B_8V20A_CM = (2, Weapons.B_8V20A_CM)
        B_8V20A_OFP2 = (2, Weapons.B_8V20A_OFP2)
        B_8V20A_OM = (2, Weapons.B_8V20A_OM)
        B_8V20A___20_S_8KOM = (2, Weapons.B_8V20A___20_S_8KOM)
        Kh_25ML = (2, Weapons.Kh_25ML)
        KMGU_2___96_AO_2_5RT = (2, Weapons.KMGU_2___96_AO_2_5RT)
        KMGU_2___96_PTAB_2_5KO = (2, Weapons.KMGU_2___96_PTAB_2_5KO)
        FAB_250 = (2, Weapons.FAB_250)
        FAB_500_M62 = (2, Weapons.FAB_500_M62)

    class Pylon3:
        B_8V20A_CM = (3, Weapons.B_8V20A_CM)
        B_8V20A_OFP2 = (3, Weapons.B_8V20A_OFP2)
        B_8V20A_OM = (3, Weapons.B_8V20A_OM)
        B_8V20A___20_S_8KOM = (3, Weapons.B_8V20A___20_S_8KOM)
        Kh_25ML = (3, Weapons.Kh_25ML)
        KMGU_2___96_AO_2_5RT = (3, Weapons.KMGU_2___96_AO_2_5RT)
        KMGU_2___96_PTAB_2_5KO = (3, Weapons.KMGU_2___96_PTAB_2_5KO)
        FAB_250 = (3, Weapons.FAB_250)
        FAB_500_M62 = (3, Weapons.FAB_500_M62)

    class Pylon4:
        B_8V20A_CM = (4, Weapons.B_8V20A_CM)
        B_8V20A_OFP2 = (4, Weapons.B_8V20A_OFP2)
        B_8V20A_OM = (4, Weapons.B_8V20A_OM)
        B_8V20A___20_S_8KOM = (4, Weapons.B_8V20A___20_S_8KOM)
        APU_6___6_9A4172_Vikhr = (4, Weapons.APU_6___6_9A4172_Vikhr)
        KMGU_2___96_AO_2_5RT = (4, Weapons.KMGU_2___96_AO_2_5RT)
        KMGU_2___96_PTAB_2_5KO = (4, Weapons.KMGU_2___96_PTAB_2_5KO)
        FAB_250 = (4, Weapons.FAB_250)
        R_73 = (4, Weapons.R_73)
        B_13L___5_S_13_OF = (4, Weapons.B_13L___5_S_13_OF)

    pylons = {1, 2, 3, 4}

    tasks = [task.CAS, task.GroundAttack, task.PinpointStrike, task.Escort, task.AFAC, task.AntishipStrike]
    task_default = task.CAS


class Mi_24V(HelicopterType):
    id = "Mi-24V"
    height = 4.354
    width = 17.3
    length = 20.953
    fuel_max = 1704
    max_speed = 320
    chaff = 0
    flare = 192
    charge_total = 192
    chaff_charge_size = 0
    flare_charge_size = 1

    class Liveries:

        class Georgia(Enum):
            standard = "standard"

        class Ukraine(Enum):
            ukraine = "ukraine"
            Ukraine_UN = "Ukraine UN"

        class Abkhazia(Enum):
            Abkhazia = "Abkhazia"

        class Russia(Enum):
            standard_1 = "standard 1"
            standard_2__faded_and_sun_bleached = "standard 2 (faded and sun-bleached)"
            Russia_FSB = "Russia_FSB"
            Russia_MVD = "Russia_MVD"

        class SouthOssetia(Enum):
            South_Ossetia = "South Ossetia"

    class Pylon1:
        _9M114_Shturm_V___2 = (1, Weapons._9M114_Shturm_V___2)

    class Pylon2:
        B_8V20A_CM = (2, Weapons.B_8V20A_CM)
        B_8V20A_OFP2 = (2, Weapons.B_8V20A_OFP2)
        B_8V20A_OM = (2, Weapons.B_8V20A_OM)
        UB_32A___32_S_5KO = (2, Weapons.UB_32A___32_S_5KO)
        _9M114_Shturm_V___2 = (2, Weapons._9M114_Shturm_V___2)
        B_8M1___20_S_8TsM = (2, Weapons.B_8M1___20_S_8TsM)
        B_13L___5_S_13_OF = (2, Weapons.B_13L___5_S_13_OF)
        UPK_23_250 = (2, Weapons.UPK_23_250)
        Fuel_tank_Ka_50 = (2, Weapons.Fuel_tank_Ka_50)
        B_8V20A___20_S_8KOM = (2, Weapons.B_8V20A___20_S_8KOM)
        GUV_VOG = (2, Weapons.GUV_VOG)

    class Pylon3:
        B_8V20A_CM = (3, Weapons.B_8V20A_CM)
        B_8V20A_OFP2 = (3, Weapons.B_8V20A_OFP2)
        B_8V20A_OM = (3, Weapons.B_8V20A_OM)
        UB_32A___32_S_5KO = (3, Weapons.UB_32A___32_S_5KO)
        B_8V20A___20_S_8KOM = (3, Weapons.B_8V20A___20_S_8KOM)
        FAB_250 = (3, Weapons.FAB_250)
        FAB_500_M62 = (3, Weapons.FAB_500_M62)
        B_8M1___20_S_8TsM = (3, Weapons.B_8M1___20_S_8TsM)
        KMGU_2___96_AO_2_5RT = (3, Weapons.KMGU_2___96_AO_2_5RT)
        B_13L___5_S_13_OF = (3, Weapons.B_13L___5_S_13_OF)
        UPK_23_250 = (3, Weapons.UPK_23_250)
        Fuel_tank_Ka_50 = (3, Weapons.Fuel_tank_Ka_50)
        GUV_YakB_GSHP = (3, Weapons.GUV_YakB_GSHP)
        GUV_VOG = (3, Weapons.GUV_VOG)

    class Pylon4:
        B_8V20A_CM = (4, Weapons.B_8V20A_CM)
        B_8V20A_OFP2 = (4, Weapons.B_8V20A_OFP2)
        B_8V20A_OM = (4, Weapons.B_8V20A_OM)
        UB_32A___32_S_5KO = (4, Weapons.UB_32A___32_S_5KO)
        B_8V20A___20_S_8KOM = (4, Weapons.B_8V20A___20_S_8KOM)
        FAB_250 = (4, Weapons.FAB_250)
        FAB_500_M62 = (4, Weapons.FAB_500_M62)
        B_8M1___20_S_8TsM = (4, Weapons.B_8M1___20_S_8TsM)
        KMGU_2___96_AO_2_5RT = (4, Weapons.KMGU_2___96_AO_2_5RT)
        B_13L___5_S_13_OF = (4, Weapons.B_13L___5_S_13_OF)
        UPK_23_250 = (4, Weapons.UPK_23_250)
        Fuel_tank_Ka_50 = (4, Weapons.Fuel_tank_Ka_50)
        GUV_YakB_GSHP = (4, Weapons.GUV_YakB_GSHP)
        GUV_VOG = (4, Weapons.GUV_VOG)

    class Pylon5:
        B_8V20A_CM = (5, Weapons.B_8V20A_CM)
        B_8V20A_OFP2 = (5, Weapons.B_8V20A_OFP2)
        B_8V20A_OM = (5, Weapons.B_8V20A_OM)
        UB_32A___32_S_5KO = (5, Weapons.UB_32A___32_S_5KO)
        _9M114_Shturm_V___2 = (5, Weapons._9M114_Shturm_V___2)
        B_8M1___20_S_8TsM = (5, Weapons.B_8M1___20_S_8TsM)
        B_13L___5_S_13_OF = (5, Weapons.B_13L___5_S_13_OF)
        UPK_23_250 = (5, Weapons.UPK_23_250)
        Fuel_tank_Ka_50 = (5, Weapons.Fuel_tank_Ka_50)
        B_8V20A___20_S_8KOM = (5, Weapons.B_8V20A___20_S_8KOM)
        GUV_VOG = (5, Weapons.GUV_VOG)

    class Pylon6:
        _9M114_Shturm_V___2 = (6, Weapons._9M114_Shturm_V___2)

    pylons = {1, 2, 3, 4, 5, 6}

    tasks = [task.CAS, task.GroundAttack, task.Escort, task.Transport, task.AFAC, task.AntishipStrike]
    task_default = task.CAS


class Mi_8MTV2(HelicopterType):
    id = "Mi-8MT"
    flyable = True
    large_parking_slot = True
    height = 4.908
    width = 21.33
    length = 25.942
    fuel_max = 1929
    max_speed = 250
    chaff = 0
    flare = 128
    charge_total = 128
    chaff_charge_size = 0
    flare_charge_size = 1

    panel_radio = {
        1: {
            "channels": {
                1: 127.5,
                2: 135,
                4: 127,
                8: 128,
                16: 132,
                17: 138,
                9: 126,
                18: 122,
                5: 125,
                10: 133,
                20: 137,
                11: 130,
                3: 136,
                6: 121,
                12: 129,
                13: 123,
                7: 141,
                14: 131,
                19: 124,
                15: 134
            },
        },
        2: {
            "channels": {
                7: 40,
                1: 21.5,
                2: 25.7,
                4: 28,
                8: 50,
                9: 55.5,
                5: 30,
                10: 59.9,
                3: 27,
                6: 32
            },
        },
    }

    property_defaults = {
        "ExhaustScreen": True,
        "LeftEngineResource": 90,
        "RightEngineResource": 90,
        "AdditionalArmor": True,
        "CargoHalfdoor": True,
    }

    class Properties:

        class ExhaustScreen:
            id = "ExhaustScreen"

        class LeftEngineResource:
            id = "LeftEngineResource"

        class RightEngineResource:
            id = "RightEngineResource"

        class AdditionalArmor:
            id = "AdditionalArmor"

        class CargoHalfdoor:
            id = "CargoHalfdoor"

    class Liveries:

        class Georgia(Enum):
            Georgia = "Georgia"

        class Australia(Enum):
            Australia = "Australia"
            Standard = "Standard"

        class Germany(Enum):
            Germany = "Germany"
            Standard = "Standard"

        class Israel(Enum):
            Israel = "Israel"
            Standard = "Standard"

        class Norway(Enum):
            Norway = "Norway"
            Standard = "Standard"

        class Spain(Enum):
            Spain = "Spain"
            Standard = "Standard"

        class Ukraine(Enum):
            Ukraine = "Ukraine"

        class Belgium(Enum):
            Belgium = "Belgium"

        class Greece(Enum):
            Hellenic_Airforce_SAR = "Hellenic Airforce SAR"
            Hellenic_Army_Aviation = "Hellenic Army Aviation"

        class UK(Enum):
            United_Kingdom = "United Kingdom"
            Standard = "Standard"

        class Insurgents(Enum):
            Insurgents = "Insurgents"
            Standard = "Standard"

        class France(Enum):
            France_ARMY = "France ARMY"
            France_NAVY = "France NAVY"
            Standard = "Standard"

        class Abkhazia(Enum):
            Abkhazia = "Abkhazia"

        class Russia(Enum):
            Russia_VVS_Standard = "Russia_VVS_Standard"
            Russia_Aeroflot = "Russia_Aeroflot"
            Russia_Gazprom = "Russia_Gazprom"
            Russia_KazanVZ = "Russia_KazanVZ"
            Russia_LII_Gromov_RA_25546 = "Russia_LII_Gromov RA-25546"
            Russia_Police = "Russia_Police"
            Russia_UTair = "Russia_UTair"
            Russia_Vertolety_Russia = "Russia_Vertolety_Russia"
            Russia_Naryan_Mar = "Russia_Naryan-Mar"
            Russia_VVS_Grey = "Russia_VVS_Grey"
            Russia_VVS_Grey_2 = "Russia_VVS_Grey_2"
            Russia_VVS_Standard_2 = "Russia_VVS_Standard_2"
            Russia_FSB = "Russia_FSB"
            Russia_MVD_Mozdok = "Russia_MVD_Mozdok"
            Russia_MVD_Standard = "Russia_MVD_Standard"
            Russia_VVS_MA = "Russia_VVS_MA"
            Russia_UN = "Russia_UN"
            Russia_PF_Ambulance = "Russia_PF_Ambulance"
            Russia_Army_Weather = "Russia_Army_Weather"

        class Italy(Enum):
            Italy_ARMY = "Italy ARMY"
            Italy_NAVY = "Italy NAVY"
            Standard = "Standard"

        class SouthOssetia(Enum):
            South_Ossetia = "South Ossetia"

        class China(Enum):
            China_PLAAA_Camo = "China PLAAA Camo"
            China_UN = "China UN"
            China_PLAAA_White = "China PLAAA White"

        class USA(Enum):
            USA_AFG = "USA_AFG"
            Standard = "Standard"

        class Denmark(Enum):
            Denmark = "Denmark"

        class Canada(Enum):
            Canada = "Canada"
            Standard = "Standard"

        class TheNetherlands(Enum):
            Netherlands_ARMY = "Netherlands ARMY"
            Netherlands_NAVY = "Netherlands NAVY"
            Standard = "Standard"

        class Turkey(Enum):
            Turkey = "Turkey"
            Standard = "Standard"

    class Pylon1:
        B_8V20A_CM = (1, Weapons.B_8V20A_CM)
        B_8V20A_OM = (1, Weapons.B_8V20A_OM)
        B_8V20A_OFP2 = (1, Weapons.B_8V20A_OFP2)
        B_8V20A___20_S_8KOM = (1, Weapons.B_8V20A___20_S_8KOM)
        FAB_100 = (1, Weapons.FAB_100)
        SAB_100 = (1, Weapons.SAB_100)
        FAB_250 = (1, Weapons.FAB_250)
        GUV_VOG = (1, Weapons.GUV_VOG)

    class Pylon2:
        B_8V20A_CM = (2, Weapons.B_8V20A_CM)
        B_8V20A_OM = (2, Weapons.B_8V20A_OM)
        B_8V20A_OFP2 = (2, Weapons.B_8V20A_OFP2)
        UPK_23_250 = (2, Weapons.UPK_23_250)
        B_8V20A___20_S_8KOM = (2, Weapons.B_8V20A___20_S_8KOM)
        FAB_250 = (2, Weapons.FAB_250)
        FAB_500_M62 = (2, Weapons.FAB_500_M62)
        GUV_YakB_GSHP = (2, Weapons.GUV_YakB_GSHP)
        GUV_VOG = (2, Weapons.GUV_VOG)
        FAB_100 = (2, Weapons.FAB_100)
        SAB_100 = (2, Weapons.SAB_100)

    class Pylon3:
        B_8V20A_CM = (3, Weapons.B_8V20A_CM)
        B_8V20A_OM = (3, Weapons.B_8V20A_OM)
        B_8V20A_OFP2 = (3, Weapons.B_8V20A_OFP2)
        B_8V20A___20_S_8KOM = (3, Weapons.B_8V20A___20_S_8KOM)
        FAB_250 = (3, Weapons.FAB_250)
        FAB_100 = (3, Weapons.FAB_100)
        SAB_100 = (3, Weapons.SAB_100)

    class Pylon4:
        B_8V20A_CM = (4, Weapons.B_8V20A_CM)
        B_8V20A_OM = (4, Weapons.B_8V20A_OM)
        B_8V20A_OFP2 = (4, Weapons.B_8V20A_OFP2)
        B_8V20A___20_S_8KOM = (4, Weapons.B_8V20A___20_S_8KOM)
        FAB_250 = (4, Weapons.FAB_250)
        FAB_100 = (4, Weapons.FAB_100)
        SAB_100 = (4, Weapons.SAB_100)

    class Pylon5:
        B_8V20A_CM = (5, Weapons.B_8V20A_CM)
        B_8V20A_OM = (5, Weapons.B_8V20A_OM)
        B_8V20A_OFP2 = (5, Weapons.B_8V20A_OFP2)
        UPK_23_250 = (5, Weapons.UPK_23_250)
        B_8V20A___20_S_8KOM = (5, Weapons.B_8V20A___20_S_8KOM)
        FAB_250 = (5, Weapons.FAB_250)
        FAB_500_M62 = (5, Weapons.FAB_500_M62)
        GUV_YakB_GSHP = (5, Weapons.GUV_YakB_GSHP)
        GUV_VOG = (5, Weapons.GUV_VOG)
        FAB_100 = (5, Weapons.FAB_100)
        SAB_100 = (5, Weapons.SAB_100)

    class Pylon6:
        B_8V20A_CM = (6, Weapons.B_8V20A_CM)
        B_8V20A_OM = (6, Weapons.B_8V20A_OM)
        B_8V20A_OFP2 = (6, Weapons.B_8V20A_OFP2)
        B_8V20A___20_S_8KOM = (6, Weapons.B_8V20A___20_S_8KOM)
        FAB_100 = (6, Weapons.FAB_100)
        SAB_100 = (6, Weapons.SAB_100)
        FAB_250 = (6, Weapons.FAB_250)
        GUV_VOG = (6, Weapons.GUV_VOG)

    class Pylon7:
        KORD_12_7 = (7, Weapons.KORD_12_7)

    class Pylon8:
        PKT_7_62 = (8, Weapons.PKT_7_62)

    pylons = {1, 2, 3, 4, 5, 6, 7, 8}

    tasks = [task.CAS, task.GroundAttack, task.Transport, task.AFAC, task.AntishipStrike]
    task_default = task.Transport


class Mi_26(HelicopterType):
    id = "Mi-26"
    group_size_max = 1
    large_parking_slot = True
    height = 12.9
    width = 32
    length = 40.854
    fuel_max = 9600
    max_speed = 300
    chaff = 0
    flare = 192
    charge_total = 192
    chaff_charge_size = 0
    flare_charge_size = 1

    class Liveries:

        class Georgia(Enum):
            United_Nations = "United Nations"

        class Syria(Enum):
            United_Nations = "United Nations"

        class Finland(Enum):
            United_Nations = "United Nations"

        class Australia(Enum):
            United_Nations = "United Nations"

        class Germany(Enum):
            United_Nations = "United Nations"

        class SaudiArabia(Enum):
            United_Nations = "United Nations"

        class Israel(Enum):
            United_Nations = "United Nations"

        class Croatia(Enum):
            United_Nations = "United Nations"

        class CzechRepublic(Enum):
            United_Nations = "United Nations"

        class Norway(Enum):
            United_Nations = "United Nations"

        class Romania(Enum):
            United_Nations = "United Nations"

        class Spain(Enum):
            United_Nations = "United Nations"

        class Ukraine(Enum):
            _7th_Separate_Brigade_of_AA__Kalinov = "7th Separate Brigade of AA (Kalinov)"
            United_Nations = "United Nations"

        class Belgium(Enum):
            United_Nations = "United Nations"

        class Slovakia(Enum):
            United_Nations = "United Nations"

        class Greece(Enum):
            United_Nations = "United Nations"

        class UK(Enum):
            United_Nations = "United Nations"

        class Insurgents(Enum):
            United_Nations = "United Nations"

        class Hungary(Enum):
            United_Nations = "United Nations"

        class France(Enum):
            United_Nations = "United Nations"

        class Abkhazia(Enum):
            United_Nations = "United Nations"

        class Russia(Enum):
            RF_Air_Force = "RF Air Force"
            Russia_FSB = "Russia_FSB"
            Russia_MVD = "Russia_MVD"
            United_Nations = "United Nations"

        class Sweden(Enum):
            United_Nations = "United Nations"

        class Austria(Enum):
            United_Nations = "United Nations"

        class Switzerland(Enum):
            United_Nations = "United Nations"

        class Italy(Enum):
            United_Nations = "United Nations"

        class SouthOssetia(Enum):
            United_Nations = "United Nations"

        class SouthKorea(Enum):
            United_Nations = "United Nations"

        class Iran(Enum):
            United_Nations = "United Nations"

        class China(Enum):
            China_Flying_Dragon_Aviation = "China Flying Dragon Aviation"
            United_Nations = "United Nations"

        class Pakistan(Enum):
            United_Nations = "United Nations"

        class Belarus(Enum):
            United_Nations = "United Nations"

        class NorthKorea(Enum):
            United_Nations = "United Nations"

        class Iraq(Enum):
            United_Nations = "United Nations"

        class Kazakhstan(Enum):
            United_Nations = "United Nations"

        class Bulgaria(Enum):
            United_Nations = "United Nations"

        class Serbia(Enum):
            United_Nations = "United Nations"

        class India(Enum):
            United_Nations = "United Nations"

        class USAFAggressors(Enum):
            United_Nations = "United Nations"

        class USA(Enum):
            United_Nations = "United Nations"

        class Denmark(Enum):
            United_Nations = "United Nations"

        class Egypt(Enum):
            United_Nations = "United Nations"

        class Canada(Enum):
            United_Nations = "United Nations"

        class TheNetherlands(Enum):
            United_Nations = "United Nations"

        class Turkey(Enum):
            United_Nations = "United Nations"

        class Japan(Enum):
            United_Nations = "United Nations"

        class Poland(Enum):
            United_Nations = "United Nations"

    pylons = {}

    tasks = [task.Transport]
    task_default = task.Transport


class Ka_27(HelicopterType):
    id = "Ka-27"
    height = 5.2
    width = 15.9
    length = 15.92
    fuel_max = 2616
    max_speed = 290

    class Liveries:

        class Ukraine(Enum):
            ukraine_camo_1 = "ukraine camo 1"

        class Russia(Enum):
            standard = "standard"

        class China(Enum):
            China_PLANAF = "China PLANAF"

    pylons = {}

    tasks = [task.Transport]
    task_default = task.Transport


class UH_60A(HelicopterType):
    id = "UH-60A"
    height = 4.893
    width = 16.4
    length = 18.654
    fuel_max = 1100
    max_speed = 300
    chaff = 30
    flare = 30
    charge_total = 60
    chaff_charge_size = 1
    flare_charge_size = 1

    class Liveries:

        class Georgia(Enum):
            standard = "standard"

        class Germany(Enum):
            standard = "standard"

        class Israel(Enum):
            ISRAIL_UN = "ISRAIL_UN"
            standard = "standard"

        class Norway(Enum):
            standard = "standard"

        class Spain(Enum):
            standard = "standard"

        class Ukraine(Enum):
            standard = "standard"

        class Belgium(Enum):
            standard = "standard"

        class UK(Enum):
            standard = "standard"

        class France(Enum):
            standard = "standard"

        class Abkhazia(Enum):
            standard = "standard"

        class Russia(Enum):
            standard = "standard"

        class Italy(Enum):
            standard = "standard"

        class SouthOssetia(Enum):
            standard = "standard"

        class USA(Enum):
            standard = "standard"

        class Denmark(Enum):
            standard = "standard"

        class Canada(Enum):
            standard = "standard"

        class TheNetherlands(Enum):
            standard = "standard"

        class Turkey(Enum):
            standard = "standard"

    pylons = {}

    tasks = [task.Transport]
    task_default = task.Transport


class CH_53E(HelicopterType):
    id = "CH-53E"
    large_parking_slot = True
    height = 7.83
    width = 24.08
    length = 30.18
    fuel_max = 1908
    max_speed = 310
    chaff = 60
    flare = 60
    charge_total = 120
    chaff_charge_size = 1
    flare_charge_size = 1

    class Liveries:

        class USA(Enum):
            standard = "standard"

    pylons = {}

    tasks = [task.Transport]
    task_default = task.Transport


class CH_47D(HelicopterType):
    id = "CH-47D"
    large_parking_slot = True
    height = 5.998
    width = 18.3
    length = 28.3
    fuel_max = 3600
    max_speed = 300
    chaff = 120
    flare = 120
    charge_total = 240
    chaff_charge_size = 1
    flare_charge_size = 1

    class Liveries:

        class Australia(Enum):
            Australia_RAAF = "Australia RAAF"

        class Spain(Enum):
            ch_47_green_spain = "ch-47_green spain"

        class Greece(Enum):
            Greek_Army = "Greek Army"

        class UK(Enum):
            ch_47_green_uk = "ch-47_green uk"

        class USA(Enum):
            standard = "standard"

        class TheNetherlands(Enum):
            ch_47_green_neth = "ch-47_green neth"

    pylons = {}

    tasks = [task.Transport]
    task_default = task.Transport


class SH_3W(HelicopterType):
    id = "SH-3W"
    large_parking_slot = True
    height = 6.807
    width = 18.91
    length = 30.207
    fuel_max = 2132
    max_speed = 270
    chaff = 30
    flare = 30
    charge_total = 60
    chaff_charge_size = 1
    flare_charge_size = 1

    class Liveries:

        class Georgia(Enum):
            standard = "standard"

        class Syria(Enum):
            standard = "standard"

        class Finland(Enum):
            standard = "standard"

        class Australia(Enum):
            standard = "standard"

        class Germany(Enum):
            standard = "standard"

        class SaudiArabia(Enum):
            standard = "standard"

        class Israel(Enum):
            standard = "standard"

        class Croatia(Enum):
            standard = "standard"

        class CzechRepublic(Enum):
            standard = "standard"

        class Norway(Enum):
            standard = "standard"

        class Romania(Enum):
            standard = "standard"

        class Spain(Enum):
            standard = "standard"

        class Ukraine(Enum):
            standard = "standard"

        class Belgium(Enum):
            standard = "standard"

        class Slovakia(Enum):
            standard = "standard"

        class Greece(Enum):
            standard = "standard"

        class UK(Enum):
            standard = "standard"

        class Insurgents(Enum):
            standard = "standard"

        class Hungary(Enum):
            standard = "standard"

        class France(Enum):
            standard = "standard"

        class Abkhazia(Enum):
            standard = "standard"

        class Russia(Enum):
            standard = "standard"

        class Sweden(Enum):
            standard = "standard"

        class Austria(Enum):
            standard = "standard"

        class Switzerland(Enum):
            standard = "standard"

        class Italy(Enum):
            standard = "standard"

        class SouthOssetia(Enum):
            standard = "standard"

        class SouthKorea(Enum):
            standard = "standard"

        class Iran(Enum):
            standard = "standard"

        class China(Enum):
            standard = "standard"

        class Pakistan(Enum):
            standard = "standard"

        class Belarus(Enum):
            standard = "standard"

        class NorthKorea(Enum):
            standard = "standard"

        class Iraq(Enum):
            standard = "standard"

        class Kazakhstan(Enum):
            standard = "standard"

        class Bulgaria(Enum):
            standard = "standard"

        class Serbia(Enum):
            standard = "standard"

        class India(Enum):
            standard = "standard"

        class USAFAggressors(Enum):
            standard = "standard"

        class USA(Enum):
            standard = "standard"

        class Denmark(Enum):
            standard = "standard"

        class Egypt(Enum):
            standard = "standard"

        class Canada(Enum):
            standard = "standard"

        class TheNetherlands(Enum):
            standard = "standard"

        class Turkey(Enum):
            standard = "standard"

        class Japan(Enum):
            standard = "standard"

        class Poland(Enum):
            standard = "standard"

    pylons = {}

    tasks = [task.Transport]
    task_default = task.Transport


class AH_64A(HelicopterType):
    id = "AH-64A"
    height = 4.15
    width = 14.63
    length = 17.87
    fuel_max = 1157
    max_speed = 300
    chaff = 30
    flare = 30
    charge_total = 60
    chaff_charge_size = 1
    flare_charge_size = 1

    class Liveries:

        class Israel(Enum):
            ah_64_a_green_isr = "ah-64_a_green isr"

        class Greece(Enum):
            greek_army = "greek army"

        class UK(Enum):
            ah_64_a_green_uk = "ah-64_a_green uk"

        class USA(Enum):
            standard = "standard"
            standard_dirty = "standard dirty"

        class TheNetherlands(Enum):
            ah_64_a_green_neth = "ah-64_a_green neth"

    class Pylon1:
        LAU_61___19_2_75__rockets_MK151_HE = (1, Weapons.LAU_61___19_2_75__rockets_MK151_HE)
        AGM_114K___4 = (1, Weapons.AGM_114K___4)
        LAU_61___19_2_75__rockets_MK151_HE = (1, Weapons.LAU_61___19_2_75__rockets_MK151_HE)

    class Pylon2:
        LAU_61___19_2_75__rockets_MK151_HE = (2, Weapons.LAU_61___19_2_75__rockets_MK151_HE)
        LAU_61___19_2_75__rockets_MK156_WP = (2, Weapons.LAU_61___19_2_75__rockets_MK156_WP)
        AGM_114K___4 = (2, Weapons.AGM_114K___4)
        LAU_61___19_2_75__rockets_MK151_HE = (2, Weapons.LAU_61___19_2_75__rockets_MK151_HE)

    class Pylon3:
        LAU_61___19_2_75__rockets_MK151_HE = (3, Weapons.LAU_61___19_2_75__rockets_MK151_HE)
        LAU_61___19_2_75__rockets_MK156_WP = (3, Weapons.LAU_61___19_2_75__rockets_MK156_WP)
        AGM_114K___4 = (3, Weapons.AGM_114K___4)
        LAU_61___19_2_75__rockets_MK151_HE = (3, Weapons.LAU_61___19_2_75__rockets_MK151_HE)

    class Pylon4:
        LAU_61___19_2_75__rockets_MK151_HE = (4, Weapons.LAU_61___19_2_75__rockets_MK151_HE)
        AGM_114K___4 = (4, Weapons.AGM_114K___4)
        LAU_61___19_2_75__rockets_MK151_HE = (4, Weapons.LAU_61___19_2_75__rockets_MK151_HE)

    pylons = {1, 2, 3, 4}

    tasks = [task.CAS, task.GroundAttack, task.Escort, task.AFAC, task.AntishipStrike]
    task_default = task.CAS


class AH_64D(HelicopterType):
    id = "AH-64D"
    height = 4.15
    width = 14.63
    length = 17.87
    fuel_max = 1157
    max_speed = 280
    chaff = 30
    flare = 30
    charge_total = 60
    chaff_charge_size = 1
    flare_charge_size = 1

    class Liveries:

        class Israel(Enum):
            ah_64_d_isr = "ah-64_d_isr"

        class Greece(Enum):
            greek_army = "greek army"

        class UK(Enum):
            ah_64_d_green_uk = "ah-64_d_green uk"

        class USA(Enum):
            standard = "standard"

        class TheNetherlands(Enum):
            ah_64_d_green_neth = "ah-64_d_green neth"

    class Pylon1:
        AGM_114K___4 = (1, Weapons.AGM_114K___4)
        LAU_61___19_2_75__rockets_MK151_HE = (1, Weapons.LAU_61___19_2_75__rockets_MK151_HE)
        LAU_61___19_2_75__rockets_MK151_HE = (1, Weapons.LAU_61___19_2_75__rockets_MK151_HE)

    class Pylon2:
        LAU_61___19_2_75__rockets_MK151_HE = (2, Weapons.LAU_61___19_2_75__rockets_MK151_HE)
        LAU_61___19_2_75__rockets_MK156_WP = (2, Weapons.LAU_61___19_2_75__rockets_MK156_WP)
        AGM_114K___4 = (2, Weapons.AGM_114K___4)
        LAU_61___19_2_75__rockets_MK151_HE = (2, Weapons.LAU_61___19_2_75__rockets_MK151_HE)

    class Pylon3:
        LAU_61___19_2_75__rockets_MK151_HE = (3, Weapons.LAU_61___19_2_75__rockets_MK151_HE)
        LAU_61___19_2_75__rockets_MK156_WP = (3, Weapons.LAU_61___19_2_75__rockets_MK156_WP)
        AGM_114K___4 = (3, Weapons.AGM_114K___4)
        LAU_61___19_2_75__rockets_MK151_HE = (3, Weapons.LAU_61___19_2_75__rockets_MK151_HE)

    class Pylon4:
        LAU_61___19_2_75__rockets_MK151_HE = (4, Weapons.LAU_61___19_2_75__rockets_MK151_HE)
        AGM_114K___4 = (4, Weapons.AGM_114K___4)
        LAU_61___19_2_75__rockets_MK151_HE = (4, Weapons.LAU_61___19_2_75__rockets_MK151_HE)

    pylons = {1, 2, 3, 4}

    tasks = [task.CAS, task.GroundAttack, task.Escort, task.AFAC, task.AntishipStrike]
    task_default = task.CAS


class AH_1W(HelicopterType):
    id = "AH-1W"
    height = 3.9
    width = 14.64
    length = 17.27
    fuel_max = 1250.0
    max_speed = 290
    chaff = 30
    flare = 30
    charge_total = 60
    chaff_charge_size = 1
    flare_charge_size = 1

    class Liveries:

        class Israel(Enum):
            standard = "standard"

        class USA(Enum):
            USA_X_Black = "USA X Black"
            USA_Marines = "USA Marines"
            standard = "standard"

        class Turkey(Enum):
            Turkey_1 = "Turkey 1"
            Turkey_2 = "Turkey 2"

    class Pylon1:
        AGM_114K___4 = (1, Weapons.AGM_114K___4)
        BGM_71D_Tow___4 = (1, Weapons.BGM_71D_Tow___4)
        LAU_61___19_2_75__rockets_MK151_HE = (1, Weapons.LAU_61___19_2_75__rockets_MK151_HE)
        M260_HYDRA = (1, Weapons.M260_HYDRA)

    class Pylon2:
        LAU_61___19_2_75__rockets_MK151_HE = (2, Weapons.LAU_61___19_2_75__rockets_MK151_HE)
        M260_HYDRA = (2, Weapons.M260_HYDRA)
        M260_HYDRA_WP = (2, Weapons.M260_HYDRA_WP)
        LAU_61___19_2_75__rockets_MK156_WP = (2, Weapons.LAU_61___19_2_75__rockets_MK156_WP)

    class Pylon3:
        LAU_61___19_2_75__rockets_MK151_HE = (3, Weapons.LAU_61___19_2_75__rockets_MK151_HE)
        M260_HYDRA = (3, Weapons.M260_HYDRA)
        M260_HYDRA_WP = (3, Weapons.M260_HYDRA_WP)
        LAU_61___19_2_75__rockets_MK156_WP = (3, Weapons.LAU_61___19_2_75__rockets_MK156_WP)

    class Pylon4:
        AGM_114K___4 = (4, Weapons.AGM_114K___4)
        LAU_61___19_2_75__rockets_MK151_HE = (4, Weapons.LAU_61___19_2_75__rockets_MK151_HE)
        M260_HYDRA = (4, Weapons.M260_HYDRA)
        BGM_71D_Tow___4 = (4, Weapons.BGM_71D_Tow___4)

    pylons = {1, 2, 3, 4}

    tasks = [task.CAS, task.GroundAttack, task.Escort, task.AFAC, task.AntishipStrike]
    task_default = task.CAS


class SH_60B(HelicopterType):
    id = "SH-60B"
    height = 4.893
    width = 16.4
    length = 18.654
    fuel_max = 1100
    max_speed = 240
    chaff = 30
    flare = 30
    charge_total = 60
    chaff_charge_size = 1
    flare_charge_size = 1

    class Liveries:

        class Greece(Enum):
            Hellenic_Navy = "Hellenic Navy"

        class USA(Enum):
            standard = "standard"

    class Pylon1:
        AGM_119B_Penguin = (1, Weapons.AGM_119B_Penguin)

    pylons = {1}

    tasks = [task.AntishipStrike, task.Transport]
    task_default = task.Transport


class UH_1H(HelicopterType):
    id = "UH-1H"
    flyable = True
    height = 4.41
    width = 14.7
    length = 17.62
    fuel_max = 631
    max_speed = 200
    chaff = 0
    flare = 60
    charge_total = 60
    chaff_charge_size = 0
    flare_charge_size = 1
    radio_frequency = 251

    panel_radio = {
        1: {
            "channels": {
                1: 251,
                2: 264,
                4: 256,
                8: 257,
                16: 261,
                17: 267,
                9: 255,
                18: 251,
                5: 254,
                10: 262,
                20: 266,
                11: 259,
                3: 265,
                6: 250,
                12: 268,
                13: 269,
                7: 270,
                14: 260,
                19: 253,
                15: 263
            },
        },
    }

    property_defaults = {
        "ExhaustScreen": True,
        "GunnersAISkill": 90,
        "EngineResource": 90,
    }

    class Properties:

        class ExhaustScreen:
            id = "ExhaustScreen"

        class GunnersAISkill:
            id = "GunnersAISkill"

        class EngineResource:
            id = "EngineResource"

    class Liveries:

        class Georgia(Enum):
            Georgian_AF_Camo = "Georgian AF Camo"
            Georgian_Air_Force = "Georgian Air Force"

        class Australia(Enum):
            Australia_RAAF_171_Sqn = "Australia RAAF 171 Sqn"
            Australia_RAAF_1968 = "Australia RAAF 1968"
            Australia_Royal_Navy = "Australia Royal Navy"

        class Germany(Enum):
            Luftwaffe = "Luftwaffe"

        class Israel(Enum):
            Israel_Army = "Israel Army"

        class Norway(Enum):
            _Civilian__Standard = "[Civilian] Standard"

        class Spain(Enum):
            Spanish_Army = "Spanish Army"
            Spanish_UN = "Spanish UN"

        class Ukraine(Enum):
            Ukrainian_Army = "Ukrainian Army"

        class Belgium(Enum):
            _Civilian__Standard = "[Civilian] Standard"

        class Greece(Enum):
            Greek_Army_Aviation = "Greek Army Aviation"
            Greek_Army_Aviation_Medic = "Greek Army Aviation Medic"
            Hellenic_Airforce_SAR = "Hellenic Airforce SAR"

        class UK(Enum):
            _Civilian__Standard = "[Civilian] Standard"

        class Insurgents(Enum):
            _Civilian__Standard = "[Civilian] Standard"

        class France(Enum):
            French_Army = "French Army"

        class Abkhazia(Enum):
            _Civilian__Standard = "[Civilian] Standard"

        class Russia(Enum):
            RF_Air_Force_Broken = "RF Air Force Broken"
            RF_Air_Force_Grey = "RF Air Force Grey"

        class Italy(Enum):
            Italy_15B_Stormo_S_A_R__Soccorso = "Italy 15B Stormo S.A.R -Soccorso"
            Italy_E_I__4B_Regg__ALTAIR = "Italy E.I. 4B Regg. ALTAIR"
            Italy_Marina_Militare_s_n__80951_7_20 = "Italy Marina Militare s.n. 80951 7-20"

        class SouthOssetia(Enum):
            _Civilian__Standard = "[Civilian] Standard"

        class USA(Enum):
            Army_Standard = "Army Standard"
            _Civilian__Standard = "[Civilian] Standard"
            US_ARMY_1972 = "US ARMY 1972"
            US_DOS = "US DOS"
            US_Ft__Rucker = "US Ft. Rucker"
            US_NAVY = "US NAVY"
            USA_Red_Flag = "USA Red Flag"
            USA_UN = "USA UN"
            XW_PFJ_Air_America = "XW-PFJ Air America"
            _Civilian__Medical = "[Civilian] Medical"
            _Civilian__NASA = "[Civilian] NASA"
            _Civilian__VIP = "[Civilian] VIP"

        class Denmark(Enum):
            _Civilian__Standard = "[Civilian] Standard"

        class Canada(Enum):
            Canadian_Force = "Canadian Force"

        class TheNetherlands(Enum):
            Royal_Netherlands_AF = "Royal Netherlands AF"

        class Turkey(Enum):
            Turkish_Air_Force = "Turkish Air Force"

    class Pylon1:
        M134_L = (1, Weapons.M134_L)

    class Pylon2:
        XM158_MK1 = (2, Weapons.XM158_MK1)
        XM158_MK5 = (2, Weapons.XM158_MK5)
        XM158_M151 = (2, Weapons.XM158_M151)
        XM158_M156 = (2, Weapons.XM158_M156)
        XM158_M274 = (2, Weapons.XM158_M274)
        XM158_M257 = (2, Weapons.XM158_M257)
        M261_MK151 = (2, Weapons.M261_MK151)
        M261_MK156 = (2, Weapons.M261_MK156)

    class Pylon3:
        M134_SIDE_L = (3, Weapons.M134_SIDE_L)
        M60_SIDE_L = (3, Weapons.M60_SIDE_L)

    class Pylon4:
        M134_SIDE_R = (4, Weapons.M134_SIDE_R)
        M60_SIDE_R = (4, Weapons.M60_SIDE_R)

    class Pylon5:
        XM158_MK1 = (5, Weapons.XM158_MK1)
        XM158_MK5 = (5, Weapons.XM158_MK5)
        XM158_M151 = (5, Weapons.XM158_M151)
        XM158_M156 = (5, Weapons.XM158_M156)
        XM158_M274 = (5, Weapons.XM158_M274)
        XM158_M257 = (5, Weapons.XM158_M257)
        M261_MK151 = (5, Weapons.M261_MK151)
        M261_MK156 = (5, Weapons.M261_MK156)

    class Pylon6:
        M134_R = (6, Weapons.M134_R)

    pylons = {1, 2, 3, 4, 5, 6}

    tasks = [task.CAS, task.GroundAttack, task.Transport]
    task_default = task.Transport


class Mi_28N(HelicopterType):
    id = "Mi-28N"
    height = 5.087
    width = 17.2
    length = 17.87
    fuel_max = 1500
    max_speed = 305
    chaff = 0
    flare = 128
    charge_total = 128
    chaff_charge_size = 0
    flare_charge_size = 1

    class Liveries:

        class Russia(Enum):
            night = "night"
            standard = "standard"

    class Pylon1:
        B_8V20A_CM = (1, Weapons.B_8V20A_CM)
        B_8V20A_OFP2 = (1, Weapons.B_8V20A_OFP2)
        B_8V20A_OM = (1, Weapons.B_8V20A_OM)
        _9M114_Shturm_V_x_8 = (1, Weapons._9M114_Shturm_V_x_8)
        B_8V20A___20_S_8KOM = (1, Weapons.B_8V20A___20_S_8KOM)
        FAB_250 = (1, Weapons.FAB_250)
        KMGU_2___96_AO_2_5RT = (1, Weapons.KMGU_2___96_AO_2_5RT)
        B_13L___5_S_13_OF = (1, Weapons.B_13L___5_S_13_OF)
        Fuel_tank_Ka_50 = (1, Weapons.Fuel_tank_Ka_50)
        B_8M1___20_S_8TsM = (1, Weapons.B_8M1___20_S_8TsM)
        FAB_500_M62 = (1, Weapons.FAB_500_M62)
        UPK_23_250 = (1, Weapons.UPK_23_250)
        KMGU_2___96_PTAB_2_5KO = (1, Weapons.KMGU_2___96_PTAB_2_5KO)

    class Pylon2:
        B_8V20A_CM = (2, Weapons.B_8V20A_CM)
        B_8V20A_OFP2 = (2, Weapons.B_8V20A_OFP2)
        B_8V20A_OM = (2, Weapons.B_8V20A_OM)
        FAB_250 = (2, Weapons.FAB_250)
        KMGU_2___96_AO_2_5RT = (2, Weapons.KMGU_2___96_AO_2_5RT)
        B_13L___5_S_13_OF = (2, Weapons.B_13L___5_S_13_OF)
        Fuel_tank_Ka_50 = (2, Weapons.Fuel_tank_Ka_50)
        B_8M1___20_S_8TsM = (2, Weapons.B_8M1___20_S_8TsM)
        FAB_500_M62 = (2, Weapons.FAB_500_M62)
        UPK_23_250 = (2, Weapons.UPK_23_250)
        KMGU_2___96_PTAB_2_5KO = (2, Weapons.KMGU_2___96_PTAB_2_5KO)
        B_8V20A___20_S_8KOM = (2, Weapons.B_8V20A___20_S_8KOM)

    class Pylon3:
        B_8V20A_CM = (3, Weapons.B_8V20A_CM)
        B_8V20A_OFP2 = (3, Weapons.B_8V20A_OFP2)
        B_8V20A_OM = (3, Weapons.B_8V20A_OM)
        FAB_250 = (3, Weapons.FAB_250)
        KMGU_2___96_AO_2_5RT = (3, Weapons.KMGU_2___96_AO_2_5RT)
        B_13L___5_S_13_OF = (3, Weapons.B_13L___5_S_13_OF)
        Fuel_tank_Ka_50 = (3, Weapons.Fuel_tank_Ka_50)
        B_8M1___20_S_8TsM = (3, Weapons.B_8M1___20_S_8TsM)
        FAB_500_M62 = (3, Weapons.FAB_500_M62)
        UPK_23_250 = (3, Weapons.UPK_23_250)
        KMGU_2___96_PTAB_2_5KO = (3, Weapons.KMGU_2___96_PTAB_2_5KO)
        B_8V20A___20_S_8KOM = (3, Weapons.B_8V20A___20_S_8KOM)

    class Pylon4:
        B_8V20A_CM = (4, Weapons.B_8V20A_CM)
        B_8V20A_OFP2 = (4, Weapons.B_8V20A_OFP2)
        B_8V20A_OM = (4, Weapons.B_8V20A_OM)
        FAB_250 = (4, Weapons.FAB_250)
        KMGU_2___96_AO_2_5RT = (4, Weapons.KMGU_2___96_AO_2_5RT)
        B_13L___5_S_13_OF = (4, Weapons.B_13L___5_S_13_OF)
        Fuel_tank_Ka_50 = (4, Weapons.Fuel_tank_Ka_50)
        B_8M1___20_S_8TsM = (4, Weapons.B_8M1___20_S_8TsM)
        FAB_500_M62 = (4, Weapons.FAB_500_M62)
        UPK_23_250 = (4, Weapons.UPK_23_250)
        KMGU_2___96_PTAB_2_5KO = (4, Weapons.KMGU_2___96_PTAB_2_5KO)
        _9M114_Shturm_V_x_8 = (4, Weapons._9M114_Shturm_V_x_8)
        B_8V20A___20_S_8KOM = (4, Weapons.B_8V20A___20_S_8KOM)

    pylons = {1, 2, 3, 4}

    tasks = [task.CAS, task.GroundAttack, task.Escort, task.AFAC, task.AntishipStrike]
    task_default = task.CAS


class OH_58D(HelicopterType):
    id = "OH-58D"
    height = 2.29
    width = 10.53
    length = 10.48
    fuel_max = 1100
    max_speed = 220
    chaff = 30
    flare = 30
    charge_total = 60
    chaff_charge_size = 1
    flare_charge_size = 1

    class Pylon1:
        M260_HYDRA = (1, Weapons.M260_HYDRA)
        M260_HYDRA_WP = (1, Weapons.M260_HYDRA_WP)
        AGM114x2_OH_58 = (1, Weapons.AGM114x2_OH_58)
        oh_58_brauning = (1, Weapons.oh_58_brauning)

    class Pylon2:
        M260_HYDRA_WP = (2, Weapons.M260_HYDRA_WP)
        M260_HYDRA = (2, Weapons.M260_HYDRA)
        AGM114x2_OH_58 = (2, Weapons.AGM114x2_OH_58)

    pylons = {1, 2}

    tasks = [task.AFAC, task.Transport, task.GroundAttack, task.Escort, task.AntishipStrike]
    task_default = task.AFAC


class SA342M(HelicopterType):
    id = "SA342M"
    flyable = True
    height = 3.192
    width = 10.5
    length = 11.97
    fuel_max = 416.33
    max_speed = 240
    chaff = 0
    flare = 32
    charge_total = 32
    chaff_charge_size = 0
    flare_charge_size = 1
    eplrs = True
    radio_frequency = 124

    panel_radio = {
        1: {
            "channels": {
                6: 40,
                2: 31,
                8: 42,
                3: 32,
                1: 30,
                4: 33,
                5: 34,
                7: 41
            },
        },
    }

    class Liveries:

        class Syria(Enum):
            Syria_Fictional = "Syria Fictional"

        class Germany(Enum):
            Germany_Fictional = "Germany Fictional"

        class Israel(Enum):
            Israel_Fictional = "Israel Fictional"

        class Greece(Enum):
            Greece_Cyprus_Fictional_Desert = "Greece Cyprus Fictional Desert"

        class UK(Enum):
            UK_Fictional = "UK Fictional"

        class France(Enum):
            Combat = "Combat"
            Tiger_Meet = "Tiger Meet"
            Tiger_Meet_2 = "Tiger Meet 2"
            Training = "Training"

        class Russia(Enum):
            Russia_Fictional = "Russia Fictional"

        class Serbia(Enum):
            Serbia_Fictional = "Serbia Fictional"
            Yugoslav_Fictional = "Yugoslav Fictional"

        class USA(Enum):
            US_Marines_Fictional = "US Marines Fictional"

    class Pylon1:
        HOT3_ = (1, Weapons.HOT3_)

    class Pylon2:
        HOT3 = (2, Weapons.HOT3)

    class Pylon3:
        HOT3_ = (3, Weapons.HOT3_)

    class Pylon4:
        HOT3 = (4, Weapons.HOT3)

    class Pylon5:
        Sand_Filter = (5, Weapons.Sand_Filter)

    class Pylon6:
        IR_Deflector = (6, Weapons.IR_Deflector)

    pylons = {1, 2, 3, 4, 5, 6}

    tasks = [task.CAS, task.GroundAttack, task.AFAC, task.Escort, task.Reconnaissance]
    task_default = task.CAS


class SA342L(HelicopterType):
    id = "SA342L"
    height = 3.192
    width = 10.5
    length = 11.97
    fuel_max = 416.33
    max_speed = 240
    chaff = 0
    flare = 32
    charge_total = 32
    chaff_charge_size = 0
    flare_charge_size = 1
    eplrs = True
    radio_frequency = 124

    panel_radio = {
        1: {
            "channels": {
                6: 40,
                2: 31,
                8: 42,
                3: 32,
                1: 30,
                4: 33,
                5: 34,
                7: 41
            },
        },
    }

    class Liveries:

        class Syria(Enum):
            Syria_Fictional = "Syria Fictional"

        class Germany(Enum):
            Germany_Fictional = "Germany Fictional"

        class Israel(Enum):
            Israel_Fictional = "Israel Fictional"

        class Greece(Enum):
            Greece_Cyprus_Fictional_Desert = "Greece Cyprus Fictional Desert"

        class UK(Enum):
            UK_Fictional = "UK Fictional"

        class France(Enum):
            Combat = "Combat"
            Tiger_Meet = "Tiger Meet"
            Tiger_Meet_2 = "Tiger Meet 2"
            Training = "Training"

        class Russia(Enum):
            Russia_Fictional = "Russia Fictional"

        class Serbia(Enum):
            Serbia_Fictional = "Serbia Fictional"
            Yugoslav_Fictional = "Yugoslav Fictional"

        class USA(Enum):
            US_Marines_Fictional = "US Marines Fictional"

    class Pylon1:
        pass

    class Pylon2:
        LAU_SNEB68G___8xSNEB68_EAP = (2, Weapons.LAU_SNEB68G___8xSNEB68_EAP)

    class Pylon5:
        Sand_Filter = (5, Weapons.Sand_Filter)

    class Pylon6:
        IR_Deflector = (6, Weapons.IR_Deflector)

    pylons = {1, 2, 5, 6}

    tasks = [task.CAS, task.GroundAttack, task.AFAC, task.Escort, task.Reconnaissance]
    task_default = task.CAS


class SA342Mistral(HelicopterType):
    id = "SA342Mistral"
    height = 3.192
    width = 10.5
    length = 11.97
    fuel_max = 416.33
    max_speed = 240
    chaff = 0
    flare = 32
    charge_total = 32
    chaff_charge_size = 0
    flare_charge_size = 1
    eplrs = True
    radio_frequency = 124

    panel_radio = {
        1: {
            "channels": {
                6: 40,
                2: 31,
                8: 42,
                3: 32,
                1: 30,
                4: 33,
                5: 34,
                7: 41
            },
        },
    }

    class Liveries:

        class Syria(Enum):
            Syria_Fictional = "Syria Fictional"

        class Germany(Enum):
            Germany_Fictional = "Germany Fictional"

        class Israel(Enum):
            Israel_Fictional = "Israel Fictional"

        class Greece(Enum):
            Greece_Cyprus_Fictional_Desert = "Greece Cyprus Fictional Desert"

        class UK(Enum):
            UK_Fictional = "UK Fictional"

        class France(Enum):
            Combat = "Combat"
            Tiger_Meet = "Tiger Meet"
            Tiger_Meet_2 = "Tiger Meet 2"
            Training = "Training"

        class Russia(Enum):
            Russia_Fictional = "Russia Fictional"

        class Serbia(Enum):
            Serbia_Fictional = "Serbia Fictional"
            Yugoslav_Fictional = "Yugoslav Fictional"

        class USA(Enum):
            US_Marines_Fictional = "US Marines Fictional"

    class Pylon1:
        Mistral_ = (1, Weapons.Mistral_)

    class Pylon2:
        Mistral = (2, Weapons.Mistral)

    class Pylon3:
        Mistral_ = (3, Weapons.Mistral_)

    class Pylon4:
        Mistral = (4, Weapons.Mistral)

    class Pylon5:
        Sand_Filter = (5, Weapons.Sand_Filter)

    class Pylon6:
        IR_Deflector = (6, Weapons.IR_Deflector)

    pylons = {1, 2, 3, 4, 5, 6}

    tasks = [task.AFAC, task.Escort, task.Reconnaissance]
    task_default = task.Escort


helicopter_map = {
    "Ka-50": Ka_50,
    "Ka-52": Ka_52,
    "Mi-24V": Mi_24V,
    "Mi-8MT": Mi_8MTV2,
    "Mi-26": Mi_26,
    "Ka-27": Ka_27,
    "UH-60A": UH_60A,
    "CH-53E": CH_53E,
    "CH-47D": CH_47D,
    "SH-3W": SH_3W,
    "AH-64A": AH_64A,
    "AH-64D": AH_64D,
    "AH-1W": AH_1W,
    "SH-60B": SH_60B,
    "UH-1H": UH_1H,
    "Mi-28N": Mi_28N,
    "OH-58D": OH_58D,
    "SA342M": SA342M,
    "SA342L": SA342L,
    "SA342Mistral": SA342Mistral,
}



import dcs
import sys
import os
import random
import argparse

zone_abkhazia = dcs.Polygon([dcs.Point(-187092.85714285, 460857.14285714), dcs.Point(-149378.57142856, 476285.71428571),
                             dcs.Point(-147664.28571428, 520000), dcs.Point(-175378.57142856, 599714.28571429),
                             dcs.Point(-174521.42857142, 644000), dcs.Point(-199664.28571428, 624000),
                             dcs.Point(-233949.99999999, 632857.14285714), dcs.Point(-271092.85714285, 596000)])


def main():
    aircrafts = [x for x in dcs.planes.plane_map.values() if x.flyable]
    aircrafts += [x for x in dcs.helicopters.helicopter_map.values() if x.flyable]
    aircraft_types = [x.id for x in aircrafts]

    parser = argparse.ArgumentParser(description="DCS WWII dogfight generator")
    parser.add_argument("-a", "--aircrafttype", default=dcs.planes.A_10C.id,
                        choices=aircraft_types,
                        help="Player aircraft type")
    parser.add_argument("-u", "--unhide", action="store_true", default=False, help="Show enemy pre mission")
    parser.add_argument("-t", "--terrain", choices=["caucasus", "nevada"], default='caucasus')
    parser.add_argument("-d", "--difficulty", choices=["easy", "normal", "hard", "ohno"], default='normal')
    parser.add_argument("-m", "--multiplayer", action="store_true", default=False)
    parser.add_argument("-s", "--stats", action="store_false", default=True)
    parser.add_argument("-o", "--output", help="Name and path of the generated mission", default=None)

    args = parser.parse_args()
    terrain_map = {
        "caucasus": dcs.terrain.Caucasus,
        "nevada": dcs.terrain.Nevada
    }
    difficulty_map = {
        'easy': 0.25,
        'normal': 0.5,
        'hard': 0.7,
        'ohno': 0.9
    }
    difficulty = difficulty_map[args.difficulty]

    if args.output is None:
        if args.terrain == "caucasus":
            args.output = os.path.join(os.path.expanduser("~"), "Saved Games\\DCS\\Missions\\oil_transport.miz")
            zone_enemy = zone_abkhazia
            destination_city = 'Adler'
        else:
            args.output = os.path.join(os.path.expanduser("~"),
                                       "Saved Games\\DCS.openalpha\\Missions\\oil_transport.miz")

    m = dcs.Mission(terrain_map[args.terrain]())
    m.random_weather = True
    m.random_date()
    m.random_daytime('day')

    for a in m.terrain.default_red_airports():
        a.set_red()

    for b in m.terrain.default_blue_airports():
        b.set_blue()

    city_graph = m.terrain.city_graph

    destination_node = city_graph.node(destination_city)

    # find a startnode far away enough
    start_node = random.choice(city_graph.rated_nodes_within(zone_enemy))
    while start_node.position.distance_to_point(destination_node.position) < 70000:
        start_node = random.choice(city_graph.rated_nodes_within(zone_enemy))

    # create the oil convoy
    abkhazia = m.country(dcs.countries.Abkhazia.name)
    convoy_vehicles = []
    for i in range(0, 1 + int(max(difficulty, random.random()) * 10)):
        convoy_vehicles.append(dcs.vehicles.Unarmed.Fuel_Truck_ATZ_10)

    airdef = [
        dcs.countries.Abkhazia.Vehicle.AirDefence.AAA_ZU_23_on_Ural_375,
        dcs.countries.Abkhazia.Vehicle.AirDefence.SPAAA_ZSU_23_4_Shilka
    ]
    if random.random() < difficulty:
        convoy_vehicles.append(random.choice(airdef))

    oil_convoy = m.vehicle_group_platoon(
        abkhazia,
        "Oil Convoy",
        convoy_vehicles,
        start_node.position.random_point_within(50))
    oil_convoy.hidden = not args.unhide
    oil_convoy.points[0].tasks.append(dcs.task.OptDisparseUnderFire())
    oil_convoy.set_skill(dcs.unit.Skill.from_percentage(difficulty))
    oil_convoy.formation_scattered(0, 50)
    oil_convoy.add_waypoint(start_node.position, dcs.point.PointAction.OnRoad)
    _, path = city_graph.travel(oil_convoy, start_node, destination_node, 100)

    # add light air defence around and in cities on path
    aaa_def = [[dcs.countries.Abkhazia.Vehicle.AirDefence.AAA_ZU_23_Emplacement],
               [dcs.countries.Abkhazia.Vehicle.AirDefence.SAM_SA_18_Igla_MANPADS,
                dcs.countries.Abkhazia.Vehicle.AirDefence.SAM_SA_18_Igla_comm]]
    for city in city_graph.rated_nodes_within(zone_enemy, 50):
        use_building_pos = int(min(difficulty, random.random()) * len(city.air_defence_pos_small))
        small_aaa_pos = list(city.air_defence_pos_small)
        for i in range(0, use_building_pos):
            p = small_aaa_pos.pop(random.randrange(0, len(small_aaa_pos)))
            vg = m.vehicle_group_platoon(abkhazia,
                                         city.name + " AAA #" + str(len(small_aaa_pos)),
                                         random.choice(aaa_def),
                                         p,
                                         0)
            vg.set_skill(dcs.unit.Skill.from_percentage(difficulty))
            vg.hidden = not args.unhide
            vg.formation_scattered(random.randrange(0, 360), 10)

    aaa_def += [[dcs.countries.Abkhazia.Vehicle.AirDefence.SPAAA_ZSU_23_4_Shilka,
                 dcs.countries.Abkhazia.Vehicle.Armor.ARV_BRDM_2,
                 dcs.countries.Abkhazia.Vehicle.Armor.ARV_BRDM_2]]
    for node in city_graph.nodes_within(zone_enemy):
        if random.random() < (difficulty - 0.1):
            vg = m.vehicle_group_platoon(abkhazia,
                                         node.name,
                                         random.choice(aaa_def),
                                         node.position.random_point_within(100, 30),
                                         random.randrange(0, 360))
            vg.set_skill(dcs.unit.Skill.from_percentage(difficulty))
            vg.hidden = not args.unhide

    # add a buk site if difficulty is hard or higher
    if difficulty > 0.5:
        buk_node = random.choice(city_graph.rated_nodes_within(zone_enemy, 50))
        sa11 = dcs.templates.VehicleTemplate.sa11_site(
            m,
            abkhazia,
            buk_node.position.random_point_within(80, 30),
            120,
            skill=dcs.unit.Skill.from_percentage(difficulty))
        sa11.hidden = not args.unhide

    # place player
    usa = m.country('USA')
    player_fg = None
    if args.multiplayer:
        for x in aircrafts:
            if dcs.task.CAS in x.tasks and x not in [dcs.planes.Su_33, dcs.planes.MiG_29A, dcs.planes.MiG_29S]:
                country = m.country(dcs.countries.Georgia.name) if x in [dcs.planes.Su_25T, dcs.planes.Su_25] else usa
                fg = m.flight_group_from_airport(country, x.id + " Client", x, m.terrain.senaki(), dcs.task.CAS, group_size=2)
                fg.add_runway_waypoint(m.terrain.senaki())
                fg.set_client()
    else:
        aircraft_type = [x for x in aircrafts if x.id == args.aircrafttype][0]
        player_fg = m.flight_group_from_airport(usa, "Player", aircraft_type, m.terrain.senaki(), dcs.task.CAS)
        player_fg.add_runway_waypoint(m.terrain.senaki())
        player_fg.units[0].set_player()

    notify_nodes = path[1:5]
    for i in range(0, int((1.3 - difficulty) * 3)):
        notifier_node = random.randrange(0, len(notify_nodes))
        notifier_node = city_graph.node(notify_nodes.pop(notifier_node))
        notify_zone = m.triggers.add_triggerzone(notifier_node.position, 300, hidden=False, name='notify_zone')
        notify_zone.hidden = True
        trig_notify = dcs.triggers.TriggerOnce(comment='NotifyConvoyPosition #' + str(i))
        trig_notify.rules.append(dcs.condition.PartOfGroupInZone(oil_convoy.id, notify_zone.id))
        trig_notify.actions.append(dcs.action.MessageToCoalition(
            dcs.action.Coalition.Blue, m.string('Intel just reported that the convoy just arrived at ' + notifier_node.name), 20))
        m.triggerrules.triggers.append(trig_notify)

    trig_convoy_dead = dcs.triggers.TriggerOnce(dcs.triggers.Event.Destroy, comment='Convoy dead')
    trig_convoy_dead.rules.append(dcs.condition.GroupDead(oil_convoy.id))
    trig_convoy_dead.actions.append(dcs.action.MessageToCoalition(
        dcs.action.Coalition.Blue, m.string('Excellent job!\nIntelligence reports the convoy was destroyed!\nRTB'), 15))
    m.triggerrules.triggers.append(trig_convoy_dead)

    m.forced_options.civil_traffic = dcs.forcedoptions.ForcedOptions.CivilTraffic.Low

    g = dcs.goals.Goal("convoy destroyed", score=100)
    g.rules.append(dcs.condition.GroupDead(oil_convoy.id))
    if player_fg:
        g.rules.append(dcs.condition.GroupAlive(player_fg.id))
    m.goals.add_blue(g)
    m.goals.add_offline(g)

    m.set_sortie_text("Search and destroy the oil convoy")
    m.set_description_text("""Abkhazia is selling oil to Russia to silently finance their military investments.

US and Georgia forces decided to prevent these oil transports.""")
    m.set_description_bluetask_text("""You are tasked to search and destroy a current oil convoy.

The position of the convoy is unknown, but we have several agents in Abkhazia that will look out
for the current route of the oil convoy.
Last known position will be transmitted to you while in flight.
Keep in mind that there are man pads and several AAA air defences around cities in Abkazia that are just waiting
to shoot down an American or Georgian aircraft.

Mission objectives:
  * Find the convoy
  * Destroy the convoy
  * Head back in one piece to Senaki airport.""")

    if args.stats:
        m.print_stats(m.stats())
    m.save(args.output)

    return 0

if __name__ == '__main__':
    sys.exit(main())



from .terrain import ParkingSlot
from .translation import String
from .unittype import UnitType, FlyingType, StaticType, ShipType, VehicleType
from .planes import PlaneType, A_10C
from .helicopters import HelicopterType, Ka_50
from . import mapping
import json
import copy
import math
from enum import Enum


class Skill(Enum):
    Average = "Average"
    Good = "Good"
    High = "High"
    Excellent = "Excellent"
    Random = "Random"
    Player = "Player"
    Client = "Client"

    @staticmethod
    def from_percentage(p):
        if 0 <= p < 0.25:
            return Skill.Average
        elif 0.25 <= p < 0.5:
            return Skill.Good
        elif 0.5 <= p < 0.75:
            return Skill.High
        elif 0.75 <= p:
            return Skill.Excellent
        return Skill.Random


class Unit:
    def __init__(self, _id, name=None, type=""):
        self.type = type
        self.position = mapping.Point(0, 0)
        self.heading = 0
        self.id = _id
        self.skill = Skill.Average  # type: Skill
        self.name = name if name else String()

    def load_from_dict(self, d):
        self.position = mapping.Point(d["x"], d["y"])
        self.heading = math.degrees(d["heading"])
        self.skill = Skill(d.get("skill")) if d.get("skill") else None

    def clone(self, _id):
        new = copy.copy(self)
        new.id = _id
        return new

    def dict(self):
        d = {
            "type": self.type,
            "x": self.position.x,
            "y": self.position.y,
            "heading": round(math.radians(self.heading), 13),
            "unitId": self.id,
            "name": self.name.id
        }
        if self.skill is not None:
            d["skill"] = self.skill.value
        return d

    def __repr__(self):
        return self.__class__.__name__ + '(' + str(self.dict()) + ')'


class FlyingUnit(Unit):
    def __init__(self, _id=None, name=None, _type: FlyingType=None, _country=None):
        super(FlyingUnit, self).__init__(_id, name, _type.id)
        self.unit_type = _type  # for loadout validation
        self.unit_type.load_payloads()
        self.livery_id = self.unit_type.default_livery(_country.name)
        self.parking = None
        self.psi = 0
        self.onboard_num = "010"
        self.alt = 0
        self.alt_type = "BARO"
        self.flare = _type.flare
        self.chaff = _type.chaff
        self.fuel = _type.fuel_max
        self.gun = 100
        self.ammo_type = _type.ammo_type
        self.pylons = {}
        self.callsign = None
        self.callsign_dict = {1: 1, 2: 1, 3: 1, "name": ""}
        self.speed = 0
        self.radio = None
        self.hardpoint_racks = True
        self.addpropaircraft = _type.property_defaults if _type.property_defaults else None

    def load_from_dict(self, d):
        super(FlyingUnit, self).load_from_dict(d)
        self.livery_id = d.get("livery_id")
        self.alt_type = d["alt_type"]
        self.alt = d["alt"]
        self.psi = d["psi"]
        self.speed = d["speed"]
        self.fuel = d["payload"]["fuel"]
        self.gun = d["payload"]["gun"]
        self.flare = d["payload"]["flare"]
        self.chaff = d["payload"]["chaff"]
        self.ammo_type = d["payload"].get("ammo_type")
        self.pylons = d["payload"]["pylons"]
        self.onboard_num = d["onboard_num"]
        if isinstance(d["callsign"], int):
            self.callsign = d["callsign"]
        else:
            self.callsign_dict = d["callsign"]
        self.parking = d.get("parking", None)
        if self.parking:
            self.parking = int(self.parking)
        self.radio = d.get("Radio")
        self.hardpoint_racks = d.get("hardpoint_racks", None)
        self.addpropaircraft = d.get("AddPropAircraft")
        return True

    def set_parking(self, parking_slot: ParkingSlot):
        parking_slot.unit_id = self.id
        self.parking = parking_slot.crossroad_idx

    def set_property(self, prop_name, value):
        if self.addpropaircraft is None:
            self.addpropaircraft = {}
        self.addpropaircraft[prop_name] = value

    def load_pylon(self, weapon, pylon=None):
        if pylon is None:
            pylon = weapon[0]
        if pylon not in self.unit_type.pylons:
            raise RuntimeError("Plane {pn} has no pylon {p}.".format(pn=self.unit_type.id, p=pylon))
        if weapon is None:
            return self.pylons.pop(pylon, None)
        self.pylons[pylon] = {"CLSID": weapon[1]["clsid"]}
        return True

    def store_loadout(self, filename):
        with open(filename, 'w') as loadout:
            json.dump(self.pylons, loadout)

        return True

    def load_loadout(self, filename):
        with open(filename, 'r') as loadout:
            self.pylons = json.load(loadout)

        return True

    def reset_loadout(self):
        self.pylons = {}

    def set_default_preset_channel(self, freq):
        if self.radio:
            self.radio[1]["channels"][1] = freq

    def set_radio_preset(self):
        if self.unit_type.panel_radio:
            self.radio = self.unit_type.panel_radio

    def set_player(self):
        self.skill = Skill.Player
        self.set_radio_preset()

    def set_client(self):
        self.skill = Skill.Client
        self.set_radio_preset()

    def dict(self):
        d = super(FlyingUnit, self).dict()
        d["alt"] = self.alt
        d["alt_type"] = self.alt_type
        if self.parking is not None:
            d["parking"] = self.parking
        if self.livery_id:
            d["livery_id"] = self.livery_id
        d["psi"] = self.psi
        d["onboard_num"] = self.onboard_num
        d["speed"] = round(self.speed, 13)
        if self.hardpoint_racks is not None:
            d["hardpoint_racks"] = self.hardpoint_racks
        if self.addpropaircraft is not None:
            d["AddPropAircraft"] = self.addpropaircraft
        d["payload"] = {
            "flare": self.flare,
            "chaff": self.chaff,
            "fuel": self.fuel,
            "gun": self.gun,
            "pylons": self.pylons
        }
        if self.ammo_type:
            d["payload"]["ammo_type"] = self.ammo_type
        if self.callsign:
            d["callsign"] = self.callsign
        else:
            d["callsign"] = self.callsign_dict
        if self.radio:
            d["Radio"] = self.radio
        return d


class Plane(FlyingUnit):
    def __init__(self, _id=None, name=None, _type: PlaneType=A_10C, _country=None):
        super(Plane, self).__init__(_id, name, _type, _country)


class Helicopter(FlyingUnit):
    def __init__(self, _id=None, name=None, _type: HelicopterType=Ka_50, _country=None):
        super(Helicopter, self).__init__(_id, name, _type, _country)
        self.rope_length = 15

    def load_from_dict(self, d):
        super(Helicopter, self).load_from_dict(d)
        self.rope_length = d["ropeLength"]

    def dict(self):
        d = super(Helicopter, self).dict()
        d["ropeLength"] = self.rope_length
        return d


class Vehicle(Unit):
    def __init__(self, id=None, name=None, _type="Sandbox"):
        super(Vehicle, self).__init__(id, name, _type)
        self.player_can_drive = False
        self.transportable = {"randomTransportable": False}

    def load_from_dict(self, d):
        super(Vehicle, self).load_from_dict(d)
        self.player_can_drive = d["playerCanDrive"]
        self.transportable = d["transportable"]

    def dict(self):
        d = super(Vehicle, self).dict()
        d["playerCanDrive"] = self.player_can_drive
        d["transportable"] = self.transportable
        return d


class Ship(Unit):
    def __init__(self, id=None, name=None, _type=None):
        super(Ship, self).__init__(id, name, _type.id)
        self.transportable = {"randomTransportable": False}

    def load_from_dict(self, d):
        super(Ship, self).load_from_dict(d)
        self.transportable = d["transportable"]

    def dict(self):
        d = super(Ship, self).dict()
        d["transportable"] = self.transportable
        return d


class Static(Unit):
    def __init__(self, unit_id=None, name=None, _type: UnitType=None):
        if not isinstance(_type, str):
            _id = _type.id
            _class = _type
        else:
            _id = _type
            _class = str
        super(Static, self).__init__(unit_id, name, _id)
        self.skill = None
        self.shape_name = None
        self.rate = None
        self.mass = None

        if issubclass(_class, StaticType):
            self.category = _type.category
            self.can_cargo = _type.can_cargo
            self.shape_name = _type.shape_name
            self.rate = _type.rate
            self.mass = 1000 if _type.can_cargo else None
        elif issubclass(_class, PlaneType):
            self.category = "Planes"
            self.can_cargo = False
        elif issubclass(_class, HelicopterType):
            self.category = "Helicopters"
            self.can_cargo = False
        elif issubclass(_class, ShipType):
            self.category = "Ships"
            self.can_cargo = False
        elif issubclass(_class, VehicleType):
            self.category = "Vehicles"
            self.can_cargo = False

    def load_from_dict(self, d):
        super(Static, self).load_from_dict(d)
        self.can_cargo = d["canCargo"]
        self.category = d["category"]
        self.shape_name = d["shape_name"]
        self.rate = d.get("rate")
        self.mass = d.get("mass")

    def dict(self):
        d = super(Static, self).dict()
        d["category"] = self.category
        d["canCargo"] = self.can_cargo
        if self.shape_name is not None:
            d["shape_name"] = self.shape_name
        if self.rate is not None:
            d["rate"] = self.rate
        if self.mass is not None:
            d["mass"] = self.mass
        return d
